/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;


/**
 * <p>
 * This is the extended version of FileStatistics which supports some advanced operations including
 * directory processing and reporting printing. The reason to separate this out is that we may not
 * need to rely on the directory traversing algorithm incorporated here (which is recursive).
 * </p>
 *
 * <p>
 * Configuration: This class shares the same configuration logic as FileStatistics. The constructor
 * simply propagates the namespace to the super class.
 * </p>
 *
 * <p>
 * Thread Safety: All the operations are delegated. This class does not have any states and is
 * fully dependant on FileStatistics' threading level.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class ExtendedFileStatistics extends FileStatistics {
    /**
     * <p>
     * Default constructor.
     * </p>
     */
    public ExtendedFileStatistics() {
        // empty
    }

    /**
     * <p>
     * Constructor with namespace.
     * </p>
     *
     * @param namespace the namespace to load config from
     *
     * @throws NullPointerException if namespace is null
     * @throws IllegalArgumentException if namespace is empty after trimmed
     * @throws ConfigurationException if config error occurs
     */
    public ExtendedFileStatistics(String namespace) throws ConfigurationException {
        super(namespace);
    }

    /**
     * <p>
     * Process a directory in non-recursive mode.
     * </p>
     *
     * @param directory the directory
     *
     * @throws NullPointerException if directory is null
     * @throws IllegalArgumentException if directory is not a valid directory
     * @throws IOException if io operation fails
     * @throws FileStatisticsException if processor fails
     */
    public void processDirectory(File directory) throws FileStatisticsException, IOException {
        processDirectory(directory, false);
    }

    /**
     * <p>
     * Process a directory in given mode. If it's non-recursive only the files in the directory is
     * processed, otherwise every files in sub-directories are processed.
     * </p>
     *
     * @param directory the directory
     * @param recursive whether to process recursively
     *
     * @throws NullPointerException if directory is null
     * @throws IllegalArgumentException if directory is not a valid directory
     * @throws IOException if io operation fails
     * @throws FileStatisticsException if processor fails
     */
    public void processDirectory(File directory, boolean recursive)
        throws FileStatisticsException, IOException {

        FileStatisticsHelper.checkNull(directory, "directory");

        // throw an exception if the directory is invalid
        if (!directory.isDirectory()) {
            throw new IllegalArgumentException("directory is not a valid directory");
        }

        // get a list of file
        File[] files = directory.listFiles();

        for (int i = 0; i < files.length; ++i) {
            // process each file and if recursive is true, process each sub-directory as well
            if (files[i].isFile()) {
                processFile(files[i]);
            } else {
                if (recursive) {
                    processDirectory(files[i], true);
                }
            }
        }
    }

    /**
     * <p>
     * Print the report to console.
     * </p>
     *
     * @throws StatsReportingException if some error occurs
     */
    public void printReport() throws StatsReportingException {
        System.out.println(getReporting().generateReport());
    }

    /**
     * <p>
     * Print the report to given file.
     * </p>
     *
     * @param file the file to write to
     *
     * @throws NullPointerException if file is null
     * @throws IOException if io operation fails
     * @throws StatsReportingException if rendering fails
     */
    public void printReport(File file) throws StatsReportingException, IOException {
        FileStatisticsHelper.checkNull(file, "file");

        String report = getReporting().generateReport();

        Writer writer = new FileWriter(file);
        try {
            writer.write(report);
        } finally {
            writer.close();
        }
    }
}
