/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics;

/**
 * <p>
 * Metric represents a measurement of the stats result. A metric is identified by its name. Metrics
 * with the same name will be aggregated. This class overrides equals() and hashCode() so that it
 * can be used as map keys.
 * </p>
 *
 * <p>
 * Thread Safety: This class is immutable and thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class Metric {
    /**
     * <p>
     * The name of the metric. Acts as an identifier.
     * </p>
     */
    private final String name;

    /**
     * <p>
     * The display name in the rendered result. If this is used in xml reporting the displayn name
     * should be well formed element name.
     * </p>
     */
    private final String displayName;

    /**
     * <p>
     * A description for the metric. This does not make a significance in this release.
     * </p>
     */
    private final String description;

    /**
     * <p>
     * Create a Metric instance with given name, display name and description.
     * </p>
     *
     * <p>
     * Note: if this metric will be used to generate xml report, the display name must be a valid
     * element name. Otherwise an exception will be thrown while generating the report.
     * </p>
     *
     * @param name the name of the metric
     * @param displayName the display name of the metric
     * @param description the description of the metric
     *
     * @throws NullPointerException if any argument is null
     * @throws IllegalArgumentException if any argument is empty after trimmed
     */
    public Metric(String name, String displayName, String description) {
        FileStatisticsHelper.checkEmpty(name, "name");
        FileStatisticsHelper.checkEmpty(displayName, "displayName");
        FileStatisticsHelper.checkEmpty(description, "description");
        this.name = name;
        this.displayName = displayName;
        this.description = description;
    }

    /**
     * <p>
     * Get the name of the metric.
     * </p>
     *
     * @return the name of the metric
     */
    public String getName() {
        return name;
    }

    /**
     * <p>
     * Get the display name of the metric.
     * </p>
     *
     * @return the display name of the metric
     */
    public String getDisplayName() {
        return displayName;
    }

    /**
     * <p>
     * Get the description of the metric.
     * </p>
     *
     * @return the description of the metric
     */
    public String getDescription() {
        return description;
    }

    /**
     * <p>
     * Return whether the given object equals to this instance.  If the names of 2 Metrics equals,
     * they equal to each other.
     * </p>
     *
     * @param object the object to check
     *
     * @return whether the given object equals to this instance.
     */
    public boolean equals(Object object) {
        if (object instanceof Metric) {
            return this.name.equals(((Metric) object).name);
        } else {
            return false;
        }
    }

    /**
     * <p>
     * Return the hash code of this instance. It equals to the name's hash code.
     * </p>
     *
     * @return the hash code
     */
    public int hashCode() {
        return name.hashCode();
    }
}
