/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.FileMatcher;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;


/**
 * <p>
 * FileCategoryMatcher is a template class that handles file category  (text/binary) oriented
 * matching logic. The purpose to still create subclasses for such minor functionality is about
 * the convenience in configuration.
 * </p>
 *
 * <p>
 * Thread Safety: This class is stateless and thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public abstract class FileCategoryMatcher implements FileMatcher {
    /**
     * <p>
     * Default protected constructor..
     * </p>
     */
    protected FileCategoryMatcher() {
        // empty
    }

    /**
     * <p>
     * Checks if the file is binary. If the first 1024 bytes of the given file are all within [32,
     * 127] or tab, carriage return or new line, this file is not binary. Otherwise this is a
     * binary file.
     * </p>
     *
     * @param file the file to check
     *
     * @return whether the file is binary
     *
     * @throws NullPointerException if file is null
     * @throws IOException if IO operation fails
     */
    public static boolean isBinary(File file) throws IOException {
        MatcherHelper.checkNull(file, "file");

        InputStream input = new FileInputStream(file);

        // check at most 1024 bytes
        try {
            byte[] buffer = new byte[1024];
            int num = input.read(buffer);

            for (int i = 0; i < num; ++i) {
                int ch = buffer[i];

                if (((ch < 32) || (ch > 127)) && ch != '\t' && ch != '\n' && ch != '\r') {
                    return true;
                }
            }
        } finally {
            input.close();
        }

        return false;
    }

    /**
     * <p>
     * Returns whether the given file is desired type.
     * </p>
     *
     * @param file the file to match
     *
     * @return whether the file matches
     *
     * @throws NullPointerException if file is null
     * @throws IOException if IO operation fails
     */
    public boolean match(File file) throws IOException {
        // delegate to match(boolean) method
        return match(isBinary(file));
    }

    /**
     * <p>
     * Decide whether the file category matches.
     * </p>
     *
     * @param isBinary whether the file is binary
     *
     * @return whether the category matches
     */
    protected abstract boolean match(boolean isBinary);
}
