/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics.processor.linecounter;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.file.statistics.processor.FileProcessorBase;
import com.topcoder.util.config.Property;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;


/**
 * <p>
 * CStyleLineCounter counts lines for C style source files. This implementation can handle C, C++,
 * C# or Java. It only takes braces and comma into account so it would not be very accurate in
 * most cases. Comments and string literals can be handled. It supports a single metric of "Line
 * Count".
 * </p>
 *
 * <p>
 * Configuration: It delegates configuration to base class without any addition.
 * </p>
 *
 * <p>
 * Thread Safety: This class is stateless while its base class is thread-safe (thus it is
 * thread-safe too).
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class CStyleLineCounter extends FileProcessorBase {
    /**
     * <p>
     * The name of the metric used for this processor.
     * </p>
     */
    public static final String NAME = "Line Count";

    /**
     * <p>
     * The display name of the metric used for this processor.
     * </p>
     */
    public static final String DISPLAY_NAME = "line";

    /**
     * <p>
     * The supported metric instance.
     * </p>
     */
    private static final Metric METRIC = new Metric(NAME, DISPLAY_NAME, "The number of lines");

    /**
     * <p>
     * The list containing the supported metric.
     * </p>
     */
    private static final List SUPPORTED_METRICS = Arrays.asList(new Metric[] {METRIC});

    /**
     * <p>
     * Constructor with both alias and matcher.
     * </p>
     *
     * @param alias the alias
     * @param matcher the file matcher
     *
     * @throws NullPointerException if alias or matcher is null
     * @throws IllegalArgumentException if alias is empty after trimmed
     */
    public CStyleLineCounter(String alias, FileMatcher matcher) {
        super(alias, matcher);
    }

    /**
     * <p>
     * This constructor loads configuration from the property. The non-empty alias and non-null
     * file matcher are required. Any error will be wrapped in a ConfigurationException.
     * </p>
     *
     * @param property the property to load config from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if config error occurs
     */
    public CStyleLineCounter(Property property) throws ConfigurationException {
        super(property);
    }

    /**
     * <p>
     * Return a list contains single Metric composed of the NAME, DISPLAY_NAME and a description.
     * </p>
     *
     * @return a list of metrics the processor supports
     */
    public List getSupportedMetrics() {
        return new ArrayList(SUPPORTED_METRICS);
    }

    /**
     * <p>
     * Count the lines in c style source code file.
     * </p>
     *
     * <p>
     * 1) Create a text stream. 2) Create a SimpleStateMachine. 3) Process each line with the state
     * machine. 3) Close the stream. 5) Create a StatsResult and place the line count under the
     * specified metric.
     * </p>
     *
     * @param file the file to process
     *
     * @return the line count result
     *
     * @throws NullPointerException if file is null
     * @throws IOException if io operation fails
     */
    public StatsResult process(File file) throws IOException {
        // check for null
        if (file == null) {
            throw new NullPointerException("file should not be null");
        }

        SimpleStateMachine stateMachine = new SimpleStateMachine();

        // open the file as a BufferedReader
        BufferedReader reader = new BufferedReader(new FileReader(file));
        try {
            while (true) {
                // read a line from the file, until EOF
                String line = reader.readLine();

                if (line == null) {
                    break;
                }

                // process the line by the simple state machine
                stateMachine.process(line);
            }
        } finally {
            reader.close();
        }

        // create a StatsResult instance, and set the metric
        StatsResult result = new StatsResult();
        result.setStat(METRIC, stateMachine.getLines());

        return result;
    }

    /**
     * <p>
     * This is a private static inner class to CStyleLineCounter that implements a state machine
     * that can process the source code. Refer to CS for the state macine description. Notice this
     * is an implementation recommendation. Developer can choose to improve from this start point.
     * Dropping this class is acceptable as long as the implementation of CStyleLineCounter is
     * stateless (and the clarity and/or efficiency aspects are improved).
     * </p>
     *
     * <p>
     * Thread Safety: This class is stateful (by name :) and not thread-safety. Each instance only
     * serves a single process.
     * </p>
     *
     * @author WishingBone, TCSDEVELOPER
     * @version 1.0
     */
    private class SimpleStateMachine {
        /**
         * <p>
         * Represents the line feed character.
         * </p>
         */
        private static final char LINE_FEED = '\n';

        /**
         * <p>
         * Represents the state not in comment or literal.
         * </p>
         */
        private static final int STATE_START = 1;

        /**
         * <p>
         * Represents the state after a single slash. (Input on slash would enter line comment
         * state; input on star would enter block state.)
         * </p>
         */
        private static final int STATE_SLASH = 2;

        /**
         * <p>
         * Represents the state of a star in a block comment. (Input of slash would enter start
         * state,)
         * </p>
         */
        private static final int STATE_STAR = 3;

        /**
         * <p>
         * Represents the state of back slash in single quote literal (next character is escaped).
         * </p>
         */
        private static final int STATE_SQ_BACKSLASH = 4;

        /**
         * <p>
         * Represents the state of back slash in double quote literal (next character is escaped).
         * </p>
         */
        private static final int STATE_DQ_BACKSLASH = 5;

        /**
         * <p>
         * Represents the state in single quote literal.
         * </p>
         */
        private static final int STATE_SINGLE_QUOTE = 6;

        /**
         * <p>
         * Represents the state in double quote literal.
         * </p>
         */
        private static final int STATE_DOUBLE_QUOTE = 7;

        /**
         * <p>
         * Represents the state in block comment.
         * </p>
         */
        private static final int STATE_BLOCK_COMMENT = 8;

        /**
         * <p>
         * Represents the state in line comment.
         * </p>
         */
        private static final int STATE_LINE_COMMENT = 9;

        /**
         * <p>
         * The state of this state machine. Reference the consts.
         * </p>
         */
        private int state = STATE_START;

        /**
         * <p>
         * The current line count. Initialized to 0 and increments whenever necessary.
         * </p>
         */
        private long lines = 0;

        /**
         * <p>
         * Default constructor.
         * </p>
         */
        public SimpleStateMachine() {
            // empty
        }

        /**
         * <p>
         * Process a line. It processes each character as well as a line feed.
         * </p>
         *
         * @param line the line to process
         *
         * @throws NullPointerException if line is null
         */
        public void process(String line) {
            // process each char
            for (int i = 0; i < line.length(); ++i) {
                process(line.charAt(i));
            }
            // process a line feed at last
            process(LINE_FEED);
        }

        /**
         * <p>
         * Process a character from input. Refer to CS for algorithm.
         * </p>
         *
         * @param ch the character to process
         */
        private void process(char ch) {
            switch (state) {
            case STATE_START:

                if (ch == '/') {
                    state = STATE_SLASH;
                } else if ((ch == ';') || (ch == '{')) {
                    lines++;
                } else if (ch == '\'') {
                    state = STATE_SINGLE_QUOTE;
                } else if (ch == '\"') {
                    state = STATE_DOUBLE_QUOTE;
                }

                break;

            case STATE_SLASH:

                if (ch == '*') {
                    state = STATE_BLOCK_COMMENT;
                } else if (ch == '/') {
                    state = STATE_LINE_COMMENT;
                } else {
                    state = STATE_START;
                }

                break;

            case STATE_STAR:

                if (ch == '/') {
                    state = STATE_START;
                } else if (ch != '*') {
                    state = STATE_BLOCK_COMMENT;
                }

                break;

            case STATE_SQ_BACKSLASH:
                state = STATE_SINGLE_QUOTE;

                break;

            case STATE_DQ_BACKSLASH:
                state = STATE_DOUBLE_QUOTE;

                break;

            case STATE_SINGLE_QUOTE:

                if (ch == '\\') {
                    state = STATE_SQ_BACKSLASH;
                } else if (ch == '\'') {
                    state = STATE_START;
                }

                break;

            case STATE_DOUBLE_QUOTE:

                if (ch == '\\') {
                    state = STATE_DQ_BACKSLASH;
                } else if (ch == '"') {
                    state = STATE_START;
                }

                break;

            case STATE_LINE_COMMENT:

                if (ch == LINE_FEED) {
                    state = STATE_START;
                }

                break;

            case STATE_BLOCK_COMMENT:

                if (ch == '*') {
                    state = STATE_STAR;
                }

                break;

            default:

            }
        }

        /**
         * <p>
         * Get the line count so far.
         * </p>
         *
         * @return the line count
         */
        public long getLines() {
            return lines;
        }
    }
}
