/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics;

import java.io.File;
import java.io.IOException;

/**
 * <p>
 * The ArchiveExtractor interface summarizes the necessary contract to extract an archive file's contents. It is
 * attached with a file matcher in order to identify the files it can extract, as well as a path separator to add to
 * the virtual path of the extracted files. Thereotically this path separator can be different for each extractor
 * in order to give a clearer illustration of different archives.
 * </p>
 * <p> Thread Safety: 
 * Implementations are advised to be stateless and thread-safe. But it's not a requirement.
 * </p>
 * 
 * 
 * @author WishingBone, TCSDEVELOPER
 * @version 2.0c
 * @since 2.0c
 */
public interface ArchiveExtractor {
	
	/**
	 * <p>
	 * Get the file matcher attached with the ArchiveExtractor in order to identify the files the extractor can handle.
	 * </p>
	 * 
	 * @return the file matcher to use
	 */
    public FileMatcher getFileMatcher();

    /**
	 * <p>
	 * Extract the archive file from source into the target directory. Relative paths should be maintained. It is caller's
	 * own responsibility to ensure the target path is writable and has enough spaces to accomodate the extracted files.
	 * </p>
	 * 
	 * 
	 * @param source the source archive file
	 * @param target the target directory
	 * @throws NullPointerException if either source or target is null
	 * @throws IOException if io fails (including source/target not exist or target is not a directory)
	 * @throws ArchiveExtractingException to encapsulate any implementation specific errors
	 */
    public void extract(File source, File target) throws IOException, ArchiveExtractingException;
    
	/**
	 * <p>
	 * Get the path separator to start the virtual path for the files extracted from this extractor.
	 * </p>
	 * 
	 * @return the path separator
	 */
    public char getPathSeparator();
}


