/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;

import com.topcoder.file.TCSFile;
import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.Property;
import com.topcoder.util.config.UnknownNamespaceException;


/**
 * <p>
 * This is the extended version of FileStatistics which supports some advanced operations including
 * directory processing and reporting printing. The reason to separate this out is that we may not
 * need to rely on the directory traversing algorithm incorporated here (which is recursive).
 * </p>
 *
 * <p>
 * Configuration: This class shares the same configuration logic as FileStatistics. The constructor
 * simply propagates the namespace to the super class.
 * </p>
 *
 * <p>
 * Thread Safety: All the operations are delegated. This class does not have any states and is
 * fully dependant on FileStatistics' threading level.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class ExtendedFileStatistics extends FileStatistics {
	
	/**
	 * <p>
	 * The property which is a container for a list of extractors. Each sub property should have another subproperty
	 * "classname" to specify the ArchiveExtractor. It is optional.
	 * </p>
	 * 
	 * 
	 * @since 2.0c
	 */
	public static final String EXTRACTORS_PROPERTY_KEY = "extractors";

	/**
	 * <p>
	 * This property specifies the fully qualified class name of the ArchiveExtractor implementation. The class should
	 * have a public constructor that accepts a Property. It is required.
	 * </p>
	 * 
	 * 
	 * @since 2.0c
	 */
	public static final String CLASSNAME_PROPERTY_KEY = "classname";

	/**
	 * <p>
	 * This property specifies the temp path to use in extracting. It's optional, but if exists, it should be a valid
	 * directory.
	 * </p>
	 * 
	 * 
	 * @since 2.0c
	 */
	public static final String TEMP_PATH_PROPERTY_KEY = "temp_path";

	/**
	 * <p>
	 * The key to retreive the defualt temp directory.
	 * </p>
	 * 
	 * @since 2.0c
	 */
	private static final String JAVA_IO_TMPDIR = "java.io.tmpdir";	
	
	
	/**
	 * <p>
	 * A list of ArchiveExtractor instances used to extract archive files. The ArchiveExtractors are tried in order
	 * and the first one is used. If none applicable the file is processed as a normal file.
	 * </p>
	 * 
	 * 
	 * @since 2.0c
	 */
	private List extractors = new ArrayList();

	/**
	 * <p>
	 * The temp path used to extract the archive files. This can be null and if it is null the default temp path is used.
	 * </p>
	 * 
	 * 
	 * @since 2.0c
	 */
    private File tempPath = null;
    
    /**
     * The id for a new tempdirectory.
     * @since 2.0c
     */
    private long tempDirectoryId = 0;
	
    /**
     * <p>
     * Default constructor.
     * </p>
     */
    public ExtendedFileStatistics() {
        // empty
    }

    /**
     * <p>
     * Constructor with namespace.
     * </p>
     *
     * @param namespace the namespace to load config from
     *
     * @throws NullPointerException if namespace is null
     * @throws IllegalArgumentException if namespace is empty after trimmed
     * @throws ConfigurationException if config error occurs
     */
    public ExtendedFileStatistics(String namespace) throws ConfigurationException {
        super(namespace);
        initializeExtractors(namespace);    
        
        Property pathProperty = getProperty(namespace, TEMP_PATH_PROPERTY_KEY, false);

        if (pathProperty != null) {
        	this.setTempPath(new File(pathProperty.getValue()));
        }
                
        
    }
    
    /**
     * Retrieves extractor from the given namespace.
     *
     * @param namespace the config namespace
     *
     * @throws ConfigurationException if some error occurs
     */
    private void initializeExtractors(String namespace)
        throws ConfigurationException {
        Property processorsProperty = getProperty(namespace, EXTRACTORS_PROPERTY_KEY, true);
        Enumeration processorsEnum = processorsProperty.propertyNames();

        // for each sub-property create a FileProcessor insatnce
        while (processorsEnum.hasMoreElements()) {
            String name = (String) processorsEnum.nextElement();
            Property processorProperty = processorsProperty.getProperty(name);

            // delegate to createObject method, and add the new processor
            this.addArchiveExtractor((ArchiveExtractor) createObject(processorProperty,
            		ArchiveExtractor.class));
        }
    }


    /**
     * Retrieve single string value of the specified sub-property. Some necessary validations are
     * performed.
     *
     * @param property the property to retrieve
     * @param key the name of the sub-property
     *
     * @return the sub-Property instance
     *
     * @throws NullPointerException if the property is null
     * @throws ConfigurationException if property does not hold a patterns sub-property.
     */
    private static String getPropertyString(Property property, String key)
        throws ConfigurationException {
        // get the subproperty
        Property subproperty = property.getProperty(key);

        if (subproperty == null) {
            throw new ConfigurationException(key + " property does not exits");
        }

        // get the string value
        String value = subproperty.getValue();

        // throw an exception if the value does not exist.
        if (value == null) {
            throw new ConfigurationException(key + " property does not contain any value.");
        }

        if (value.trim().length() == 0) {
            throw new ConfigurationException(key + " property is an empty string.");
        }

        return value;
    }

    /**
     * Get the specified property from given namespace.
     *
     * @param namespace the config namespace
     * @param key the property key
     * @param required whether this property is required
     *
     * @return the property retrieved
     *
     * @throws ConfigurationException if the namespace is invalid or the property does not exist if
     *         it's required
     */
    private Property getProperty(String namespace, String key, boolean required)
        throws ConfigurationException {
        Property property = null;

        try {
            property = ConfigManager.getInstance().getPropertyObject(namespace, key);
        } catch (UnknownNamespaceException une) {
            throw new ConfigurationException("the given namespace does not exits", une);
        }

        if ((property == null) && required) {
            throw new ConfigurationException(PROCESSORS_PROPERTY_KEY + " property does not exist");
        }

        return property;
    }    

    /**
     * <p>
     * Process a directory in non-recursive mode.
     * </p>
     *
     * @param directory the directory
     *
     * @throws NullPointerException if directory is null
     * @throws IllegalArgumentException if directory is not a valid directory
     * @throws IOException if io operation fails
     * @throws FileStatisticsException if processor fails
     */
    public void processDirectory(File directory) throws FileStatisticsException, IOException {
        processDirectory(directory, false);
    }

    /**
     * <p>
     * Process a directory in given mode. If it's non-recursive only the files in the directory is
     * processed, otherwise every files in sub-directories are processed.
     * </p>
     *
     * @param directory the directory
     * @param recursive whether to process recursively
     *
     * @throws NullPointerException if directory is null
     * @throws IllegalArgumentException if directory is not a valid directory
     * @throws IOException if io operation fails
     * @throws FileStatisticsException if processor fails
     */
    public void processDirectory(File directory, boolean recursive)
        throws FileStatisticsException, IOException {
    	if (recursive) {
    		processDirectory(directory, Integer.MAX_VALUE);
    	} else {    		
    		processDirectory(directory, 0);    		
    	}        
    }

    /**
     * <p>
     * This method is added since version 2.0c to process a directory (recursively) and specifies how many nesting
     * level of archives should be extracted. It delegates to processDirectory(directory, directory.path, nestingLevel).
     * </p>
     * 
     * 
     * @param directory the directory
     * @param nestingLevel the max nesting level
     * @throws NullPointerException if directory is null
     * @throws IllegalArgumentException if nestingLevel is negative
     * @throws IOException if io operation fails
     * @throws FileStatisticsException if processor fails
     * @since 2.0c
     */
    public void processDirectory(File directory, int nestingLevel) throws IOException, FileStatisticsException { 
    	processDirectory(directory, directory.getAbsolutePath() + File.separator, nestingLevel);
    }

    /**
     * <p>
     * Process a directory with a max nesting level specified.
     * </p>
     * <algorithm>
     * This is the core of the new functionality added in version 2.0c:
     * 1) It is advised to process recursively with implementation.
     * 2) The virtual path should be used with aggregation.
     * 3) When a file is processed, all the extractors should be tried in the order they are configured.
     * 4) If a file matches an extractor, it will not be procsesed by FileStatistics.
     * 5) Before extracting the file, a temp path should be created within the given temp path (name is not important).
     * 6) The path separator from the ArcniveExtractor should be used to separate the new virtual path from the
     * old one.
     * 7) Whatever happens the created temp path should be removed after the process with deltree form TCSFile,
     * unless deltree errors out.
     * </algorithm>
     * 
     * 
     * @param directory the directory to process
     * @param virtualPath the virtual path to aggregate to
     * @param nestingLevel the max nesting level
     * @throws NullPointerException if directory or virtualPath is null
     * @throws IllegalArgumentException if nestingLevel is negative or virtualPath is empty after trimmed
     * @throws IOException if io operation fails
     * @throws FileStatisticsException if processor fails
     * @since 2.0c
     */
    protected void processDirectory(File directory, String virtualPath, int nestingLevel) 
    	throws IOException, FileStatisticsException {  
    	if (directory == null) {
    		throw new NullPointerException("directory should not be null");
    	}
    	if (virtualPath == null) {
    		throw new NullPointerException("virtualPath should not be null");    		
    	}
    	if (virtualPath.trim().length() == 0) {
    		throw new IllegalArgumentException("virtualPath should not be empty");
    	}
    	if (nestingLevel < 0) {
    		throw new NullPointerException("nestingLevel should not be negative");    		
    	}    	
    	
    	// throw an exception if the directory is invalid
        if (!directory.isDirectory()) {
            throw new IllegalArgumentException("directory is not a valid directory");
        }

        // get a list of file
        File[] files = directory.listFiles();

        for (int i = 0; i < files.length; ++i) {
        	String filePath = virtualPath + files[i].getName();
        	if (files[i].isFile()) {        		
                boolean extractorMatched = false;
                if (nestingLevel > 0) {
	        		Iterator itr = extractors.iterator();
		        	while (itr.hasNext()) {
		        		ArchiveExtractor extractor = (ArchiveExtractor) itr.next();
		        		if (extractor.getFileMatcher().match(files[i])) {
		        			File tempDirectory = createTempDirectory(files[i].getName());		        			
		        			try {
			        			extractor.extract(files[i], tempDirectory);
			        			
			        			processDirectory(tempDirectory, filePath + extractor.getPathSeparator(), nestingLevel - 1) ;
			        			
		        			} finally {
		        				TCSFile tcsFile = new TCSFile(tempDirectory);		        			
		        				tcsFile.deleteTree();
		        			}
		        			extractorMatched = true;
		        			break;
		        		}
		        	}
                }
	        	if (!extractorMatched) {
	        		// process each file and if recursive is true, process each sub-directory as well
	                processFile(files[i], virtualPath + files[i].getName());	                
	                		
	        	}
            } else {
            	processDirectory(files[i], filePath + File.separator, nestingLevel);
            }
        }
    	
    	
    	
    
    }        

	/**
     * <p>
     * Print the report to console.
     * </p>
     *
     * @throws StatsReportingException if some error occurs
     */
    public void printReport() throws StatsReportingException {
        System.out.println(getReporting().generateReport());
    }

    /**
     * <p>
     * Print the report to given file.
     * </p>
     *
     * @param file the file to write to
     *
     * @throws NullPointerException if file is null
     * @throws IOException if io operation fails
     * @throws StatsReportingException if rendering fails
     */
    public void printReport(File file) throws StatsReportingException, IOException {
        FileStatisticsHelper.checkNull(file, "file");

        String report = getReporting().generateReport();

        Writer writer = new FileWriter(file);
        try {
            writer.write(report);
        } finally {
            writer.close();
        }
    }
    
    /**
     * <p>
     * Add the extractor to the end of the list (do not care about duplication).
     * </p>
     * 
     * 
     * @return the position of the added extractor
     * @param extractor the extractor to add
     * @throws NullPointerException if extractor is null
     * @since 2.0c
     */
    public int addArchiveExtractor(ArchiveExtractor extractor) {
    	if (extractor == null) {
    		throw new NullPointerException("extractor should not be null");
    	}
    	extractors.add(extractor);
    	return extractors.size() - 1;
    }

    /**
     * <p>
     * Remove the extractor from given position.
     * </p>
     * 
     * 
     * @return the removed extractor
     * @param index the index of the extractor to remove
     * @throws IndexOutOfBoundsException if index is not valid
     * @since 2.0c
     */
    public ArchiveExtractor removeArchiveExtractor(int index) {
    	return (ArchiveExtractor) extractors.remove(index);
    }

    /**
     * <p>
     * Get a shallow copy of the extractor list.
     * </p>
     * 
     * @return a list of extractors
     * @throws 2.0c
     */
    public List getAllArchiveExtractors() {
    	return new ArrayList(extractors);	   
    }
   
    /**
     * <p>
     * Clear all extractors.
     * </p>
     * 
     * @since 2.0c
     */
    public void clearAllArchiveExtractors() {
    	extractors.clear();
    }

    /**
     * <p>
     * Get the temp path.
     * </p>
     * 
     * @return the temp path
     * @since 2.0c
     */
    public File getTempPath() {
    	return tempPath;    	
    }

    /**
     * <p>
     * Set the temp path which is nullable.
     * </p>
     * 
     * @param tempPath the temp path to set
     * @throws IllegalArgumentException if tempPath is not a valid directory
     * @since 2.0c
     */
    public void setTempPath(File tempPath) {
    	
    	if (tempPath != null) {
    		// check whether the path is a valid directory
    		if (!tempPath.isDirectory()) {
    			throw new IllegalArgumentException("tempPath is not a valid dir");
    		}    		
    	}
    	
    	this.tempPath = tempPath;
    }    
    
    /**
     * Create a temp dir.
     * @return teh dir
     * @throws IOException if io fails.
     */
    private File createTempDirectory(String dirName) throws IOException {
    	// append a unique id for the dirName in order to avoid duplicate directory names
    	synchronized (this) {
    		dirName = dirName + tempDirectoryId;
    		++tempDirectoryId; 
    	}
    	File dir = null;
    	if (tempPath == null) {
    		
    		dir = new File(System.getProperty(JAVA_IO_TMPDIR), dirName);         	
    	} else {
    		dir = new File(tempPath, dirName);
    	}
    	dir.mkdir();
    	return dir;
    }
}
