/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics.extractor;

import java.lang.reflect.Constructor;

import com.topcoder.file.statistics.ArchiveExtractor;
import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.matcher.AnyFileMatcher;
import com.topcoder.util.config.Property;
									


/**
 * <p>
 * An implementation base for ArchiveExtractor which implements the logic to load configuration for the file
 * matcher and path separator.
 * </p>
 * <p>
 * Configuration:
 * 1) "matcher" which acts as a container for the "classname" of the FileMatcher implementation as well as
 * other potential configuration for the FileMatcher.
 * 2) "path_separator" contains single character to start the virtual path extracted from this extractor.
 * </p>
 * <p> 
 * Thread Safety:
 * This class itself is thread-safe.
 * </p>
 * 
 * 
 * @author WishingBone, TCSDEVELOPER
 * @version 2.0c
 * @since 2.0c
 */
public abstract class ArchiveExtractorBase implements ArchiveExtractor {

	/**
	 * <p>
	 * This property is a container for the class name of the FileMatcher implementation as well as other configuration
	 * for the FileMatcher implementation to configure from. This property is required.
	 * </p>
	 */
    public static final String FILE_MATCHER_PROPERTY_KEY = "matcher";

	/**
	 * <p>
	 * The fully qualified class name of the FileMatcher implementation which should have a constructor that
	 * accepts a Property. This property is required.
	 * </p>
	 */
    public static final String CLASSNAME_PROPERTY_KEY = "classname";

	/**
	 * <p>
	 * The path separator to use with this ArchiveExtractor. It is optional (defaulting to '>'). If it exists it should have
	 * a single character.
	 * </p>
	 */
    public static final String PATH_SEPARATOR_PROPERTY_KEY = "path_separator";
    
    /**
     * <p>
     * Constant for the parameter types to retrieve the constructor.
     * </p>
     */
    private static final Class[] PARAMETER_TYPES = new Class[] {Property.class};


	/**
	 * <p>
	 * The matcher for the ArchiveExtractor that specifies the files can be handled. It is set in constructor and never
	 * changes.
	 * </p>
	 */
    private final FileMatcher matcher;

	/**
	 * <p>
	 * The path separator to use with the ArchiveExtractor. It is set in constructor and never changes.
	 * </p>
	 */
    private char pathSeparator = '>';

	/**
	 * <p>
	 * Constructor with file matcher and path separator.
	 * </p>
	 * 
	 * 
	 * @param matcher the file matcher
	 * @param pathSeparator the path separator
	 * @throws NullPointerException if matcher is null
	 */
    protected ArchiveExtractorBase(FileMatcher matcher, char pathSeparator) {        
    	// check for null
    	if (matcher == null) {
    		throw new NullPointerException("matcher should not be null");
    	}
    	
    	// initialize the inner fields
    	this.matcher = matcher;
    	this.pathSeparator = pathSeparator;
    	
    } 

	/**
	 * <p>
	 * The constructor loads configuration from property node.
	 * </p>
	 * 
	 * 
	 * @param property the property to configure from
	 * @throws NullPointerException if property is null
	 * @throws ConfigurationException if config error occurs
	 */
    protected ArchiveExtractorBase(Property property) throws ConfigurationException {  
    	
    	this.matcher = createMatcher(getProperty(property, FILE_MATCHER_PROPERTY_KEY, true));
    	
    	Property separatorPorperty = getProperty(property, PATH_SEPARATOR_PROPERTY_KEY, false);
    	if (separatorPorperty != null) {
    		String separator = separatorPorperty.getValue();
    		if (separator.length() > 1) {
    			throw new ConfigurationException(separator + " is invalid separator");
    		}
    		this.pathSeparator = separator.charAt(0);
    	}        
    	
    } 

	/**
	 * <p>
	 * Get the file matcher.
	 * </p>
	 * 
	 * @return the file matcher
	 */
    public FileMatcher getFileMatcher() {        
        return matcher;
    } 

	/**
	 * <p>
	 * Get the path separator.
	 * </p>
	 *
	 * @return the path separator
	 */
    public char getPathSeparator() {        
        return pathSeparator;
    } 
    
    /**
     * Retrieve specified sub-property from given property. Some necessary validations are
     * performed.
     *
     * @param property the property to retrieve
     * @param key the name of the sub-property
     * @param required whether this property is required
     *
     * @return the sub-Property instance
     *
     * @throws NullPointerException if the property is null
     * @throws ConfigurationException if property does not hold a patterns sub-property.
     */
    private static Property getProperty(Property property, String key, boolean required)
        throws ConfigurationException {
        // check for null
        if (property == null) {
            throw new NullPointerException("property should not be null.");
        }

        Property subproperty = property.getProperty(key);

        // check whether the sub-property exists
        if (subproperty == null && required) {
            throw new ConfigurationException("property does not contain the given sub-property - " + key);
        }

        return subproperty;
    }

    /**
     * Retrieve single string value of the specified sub-property. Some necessary validations are
     * performed.
     *
     * @param property the property to retrieve
     * @param key the name of the sub-property
     *
     * @return the sub-Property instance
     *
     * @throws NullPointerException if the property is null
     * @throws ConfigurationException if property does not hold a patterns sub-property.
     */
    private static String getPropertyString(Property property, String key)
        throws ConfigurationException {
        // get the subproperty
        Property subproperty = getProperty(property, key, true);

        // get the string value
        String value = subproperty.getValue();

        // throw an exception if the value does not exist.
        if (value == null) {
            throw new ConfigurationException(key + " property does not contain any value.");
        }

        if (value.trim().length() == 0) {
            throw new ConfigurationException(key + " property is an empty string.");
        }

        return value;
    }
    
    /**
     * Creates a FileMatcher instance from given property via reflection.
     *
     * @param property the property to retrieve
     *
     * @return the sub-Property instance
     *
     * @throws NullPointerException if the property is null
     * @throws ConfigurationException if property does not hold a patterns sub-property.
     */
    private static FileMatcher createMatcher(Property property)
        throws ConfigurationException {

        if (property == null) {
            return new AnyFileMatcher();
        }

        // get the class name
        String className = getPropertyString(property, CLASSNAME_PROPERTY_KEY);

        try {
            // retrieve the public constructor by the parameter type array
            Constructor matcherConstructor = Class.forName(className).getConstructor(PARAMETER_TYPES);

            // create the matcher instance via reflection
            return (FileMatcher) matcherConstructor.newInstance(new Object[] {property});
        } catch (Exception e) {
            // wrap the exception with ConfigurationException
            throw new ConfigurationException("Exception occurs while creating the instance", e);
        }
    }    
 }
