/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics.reporting;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsReportingException;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.util.config.Property;
import java.io.File;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.Writer;
import java.util.Iterator;
import java.util.List;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;


/**
 * <p>
 * XmlStatsReporting is a reporting implementation that renders XML reports. Refer to component
 * specification for formatting of the XML. It is capable of using a stylesheet to transform the
 * XML into user-oriented format (HTML, CSV, etc.)
 * </p>
 *
 * <p>
 * Configuration: It supports a single property "stylesheet" which contains a filename of the
 * stylesheet. This property is optional. If it's missing the raw XML is generated.
 * </p>
 *
 * <p>
 * Thread Safety: This class is not thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class XmlStatsReporting extends com.topcoder.file.statistics.reporting.AbstractStatsReporting {
    /**
     * <p>
     * The property which specifies a stylesheet to apply on the xml. If this property is not
     * specified, the raw output is generated.
     * </p>
     */
    public static final String STYLESHEET_PROPERTY_KEY = "stylesheet";

    /**
     * <p>
     * Line separator character.
     * </p>
     */
    private static final char LINE_SEPARATOR = (char) Character.LINE_SEPARATOR;

    /**
     * The name of the report element.
     */
    private static final String REPORT_ELEMENT = "report";

    /**
     * The name of the files element.
     */
    private static final String FILES_ELEMENT = "files";

    /**
     * The name of the file element.
     */
    private static final String FILE_ELEMENT = "file";

    /**
     * The name of the directories element.
     */
    private static final String DIRECTORIES_ELEMENT = "directories";

    /**
     * The name of the directory element.
     */
    private static final String DIRECTORY_ELEMENT = "directory";

    /**
     * The name of the aliases element.
     */
    private static final String ALIASES_ELEMENT = "aliases";

    /**
     * The name of the alias element.
     */
    private static final String ALIAS_ELEMENT = "alias";

    /**
     * The name of the overall element.
     */
    private static final String OVERALL_ELEMENT = "overall";

    /**
     * The name of the 'name' attribute.
     */
    private static final String NAME_ATTRIBUTE = "name";

    /**
     * The indent in the generated xml.
     */
    private static final int INDENT = 4;

    /**
     * <p>
     * The stylesheet to apply on the raw xml generated. This is initialized in constructor and
     * never changed. The value can be null indicating no stylesheet is used.
     * </p>
     */
    private final File stylesheet;

    /**
     * <p>
     * Default constructor.
     * </p>
     */
    public XmlStatsReporting() {
        this.stylesheet = null;
    }

    /**
     * <p>
     * Constructor with a styleshee.
     * </p>
     *
     * @param stylesheet the stylesheet
     *
     * @throws NullPointerException if stylesheet is null
     * @throws IllegalArgumentException if stylesheet is not a valid file
     */
    public XmlStatsReporting(File stylesheet) {
        if (stylesheet == null) {
            throw new NullPointerException("stylesheet should not be null");
        }

        if (!stylesheet.isFile()) {
            throw new IllegalArgumentException("stylesheet is not a valid file");
        }

        this.stylesheet = stylesheet;
    }

    /**
     * <p>
     * Configures from a property. propagate it to super constructor. After that, it reads a String
     * property from "stylesheet" and store it to variable. This property is optional, but should
     * be an existing filename if exists.
     * </p>
     *
     * @param property the property to load config from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if config error occurs
     */
    public XmlStatsReporting(Property property) throws ConfigurationException {
    	super(property);
        // check for null
        if (property == null) {
            throw new NullPointerException("property should not be null");
        }

        // get the file name
        String fileName = property.getValue(STYLESHEET_PROPERTY_KEY);

        if (fileName == null) {
            stylesheet = null;
        } else {
            // set the stylesheet field if necessary
            stylesheet = new File(fileName);

            // check whether it's an existing file.
            if (!stylesheet.isFile()) {
                throw new ConfigurationException(
                    "the specified file does not exits or is a directory");
            }
        }
    }

    /**
     * <p>
     * This generates an XML format report if no stylesheet is specified, or outputs the result of
     * XSLT. Refer to test_files/reporting/sample-report for more details about the raw xml.
     * </p>
     *
     * @return the generated report
     *
     * @throws StatsReportingException if rendering fails
     */
    public String generateReport() throws StatsReportingException {
        String rawXml = generateRawXml();

        if (stylesheet == null) {
            // return the raw xml immediately if there is no stylesheet
            return rawXml;
        }

        // transform the xml and return
        return transform(rawXml);
    }

    /**
     * Transform the raw xml with the stylesheet.
     *
     * @param xml the raw xml to transform
     *
     * @return the transformed xml
     *
     * @throws StatsReportingException if rendering fails
     */
    private String transform(String xml) throws StatsReportingException {
        try {
            // Use the static TransformerFactory.newInstance() method to instantiate
            // a TransformerFactory.
            TransformerFactory transformerFactory = TransformerFactory.newInstance();

            // Create a new transformer
            Transformer transformer = transformerFactory.newTransformer(new StreamSource(stylesheet));

            Writer result = new StringWriter();
            transformer.transform(new StreamSource(new StringReader(xml)), new StreamResult(result));

            return result.toString();
        } catch (TransformerException te) {
            throw new StatsReportingException("Some errors occur while transforming", te);
        }
    }

    /**
     * Generate the raw XML string. See test_files/reporting/sample-report.xml.
     *
     * @return a string containing the generated raw xml
     */
    private String generateRawXml() {
        StringBuffer report = new StringBuffer();

        generateElement(REPORT_ELEMENT, null, report, 0);

        // delegate to generateStats method to render the xml
        generateStats(FILES_ELEMENT, FILE_ELEMENT, getFileStats(), report, INDENT);
        generateStats(DIRECTORIES_ELEMENT, DIRECTORY_ELEMENT, getDirectoryStats(), report, INDENT);
        generateStats(ALIASES_ELEMENT, ALIAS_ELEMENT, getAliasStats(), report, INDENT);
        generateStats(OVERALL_ELEMENT, null, getOverallStats(), report, INDENT);

        generateElement("/" + REPORT_ELEMENT, null, report, 0);

        return report.toString();
    }

    /**
     * Renders a StatsCollection with specified element name and subelement name.
     *
     * @param elementName the element name
     * @param subElementName the sub element name
     * @param stats the StatsCollection to render
     * @param report StringBuffer containing the report
     * @param indent current indent
     */
    private void generateStats(String elementName, String subElementName, StatsCollection stats,
        StringBuffer report, int indent) {
        generateElement(elementName, null, report, indent);

        List results = stats.getAllKeys();

        // render each result
        for (Iterator itr = results.iterator(); itr.hasNext();) {
            String key = (String) itr.next();
            StatsResult result = stats.get(key);
            generateStats(subElementName, key, result, report, indent + INDENT);
        }

        generateElement("/" + elementName, null, report, indent);
    }

    /**
     * Renders a StatsResult with specified element name and attribute.
     *
     * @param elementName the element name
     * @param attribute the 'name' attribute value
     * @param stats the StatsResult to render
     * @param report StringBuffer containing the report
     * @param indent current indent
     */
    private void generateStats(String elementName, String attribute, StatsResult stats,
        StringBuffer report, int indent) {
        generateElement(elementName, attribute, report, indent);

        List metrics = stats.getMetrics();

        // render each metrics
        for (Iterator itr = metrics.iterator(); itr.hasNext();) {
            Metric metric = (Metric) itr.next();
            String displayName = metric.getDisplayName();

            for (int i = 0; i < (indent + INDENT); ++i) {
                report.append(' ');
            }

            report.append("<").append(displayName).append(">");
            report.append(stats.getStat(metric));
            report.append("</").append(displayName).append(">").append(LINE_SEPARATOR);
        }

        generateElement("/" + elementName, null, report, indent);
    }

    /**
     * Renders a element with specified element name and attribute.
     *
     * @param elementName the element name
     * @param nameAttribute the 'name' attribute value
     * @param report StringBuffer containing the report
     * @param indent current indent
     */
    private void generateElement(String elementName, String nameAttribute, StringBuffer report,
        int indent) {
        // output some indent spaces
        for (int i = 0; i < indent; ++i) {
            report.append(' ');
        }

        report.append("<").append(elementName);

        // render the name attribute if necessary
        if (nameAttribute != null) {
            report.append(" ").append(NAME_ATTRIBUTE).append("=\"").append(nameAttribute).append('\"');
        }

        report.append(">").append(LINE_SEPARATOR);
    }
}
