/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics;

import com.topcoder.file.statistics.matcher.AnyFileMatcher;
import com.topcoder.file.statistics.matcher.BinaryFileMatcher;
import com.topcoder.file.statistics.matcher.ExtensionMatcher;
import com.topcoder.file.statistics.matcher.MimeMatcher;
import com.topcoder.file.statistics.matcher.RegexMatcher;
import com.topcoder.file.statistics.matcher.TextFileMatcher;
import com.topcoder.file.statistics.matcher.TypeNameMatcher;
import com.topcoder.file.statistics.processor.linecounter.CStyleLineCounter;
import com.topcoder.file.statistics.processor.linecounter.TextLineCounter;
import com.topcoder.file.statistics.reporting.BasicStatsReporting;
import com.topcoder.file.statistics.reporting.XmlStatsReporting;
import junit.framework.TestCase;
import java.io.File;
import java.util.List;
import java.util.regex.Pattern;


/**
 * <p>
 * Component demonstration for File Static.
 * </p>
 *
 * <p>
 * The File Statistics component collects statistical information for one or more files.  Usually
 * these statistics are associated with the file content (number of lines, number of occurrences
 * of a string, etc.) as opposed to the file itself (file size, last modified time, etc.) although
 * both types of statistics could potentially be calculated by the component.  The code that
 * calculates a statistic is pluggable so that many different types of statistics can be
 * collected.  Furthermore, a mechanism exists that allows each statistic plug-in to specify what
 * types of files it can or cannot handle.  For example, a plug-in that counts lines of code would
 * be applicable to .java files but not .txt files.  After all files have been examined, a report
 * is generated showing the results on a per-file, per-directory and aggregate basis.  The report
 * generation code is also pluggable and a default implementation is supplied.  Finally, the
 * component can optionally be run as a standalone application from the command line
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class Demo extends TestCase {
    /**
     * The namespace for this test.
     */
    private static final String NAMESPACE = "com.topcoder.file.statistics.unittest";

    /**
     * The default namespace.
     */
    private static final String DEFAULT_NAMESPACE = "com.topcoder.file.statistics.Main";

    /**
     * The namespace for Magic Numbers component.
     */
    private static final String MAGIC_NUMBERS_NAMESPACE = "com.topcoder.file.mime.MagicNumbers.xml";

    /**
     * Load the configuration before testing starts.
     *
     * @throws Exception to JUnit
     */
    protected void setUp() throws Exception {
        TestHelper.loadConfigFile("demo.xml", NAMESPACE);
    }

    /**
     * Remove the configuration after testing.
     *
     * @throws Exception to JUnit
     */
    protected void tearDown() throws Exception {
        TestHelper.removeNamespace(NAMESPACE);
        TestHelper.removeNamespace(DEFAULT_NAMESPACE);
        TestHelper.removeNamespace(MAGIC_NUMBERS_NAMESPACE);
    }

    /**
     * <p>
     * Demonstrates the general usage of this component.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testUsageDemo() throws Exception {
        // create a file statistics instance
        ExtendedFileStatistics statistics = new ExtendedFileStatistics();

        // create file matchers
        FileMatcher matcher1 = new AnyFileMatcher();
        FileMatcher matcher2 = new ExtensionMatcher("txt");
        FileMatcher matcher3 = new RegexMatcher(Pattern.compile("pattern"));
        FileMatcher matcher4 = new TextFileMatcher();
        FileMatcher matcher5 = new BinaryFileMatcher();
        FileMatcher matcher6 = new TypeNameMatcher("Java");
        FileMatcher matcher7 = new MimeMatcher("text/plain");

        // create file processors
        FileProcessor processor1 = new TextLineCounter("Text File", matcher1);
        FileProcessor processor2 = new CStyleLineCounter("Source File", matcher6);

        // create stats reporting implementation
        StatsReporting reporting1 = new BasicStatsReporting();
        StatsReporting reporting2 = new XmlStatsReporting(new File(
                    "test_files/reporting/xml_to_html.xsl"));

        // manipulate file processors
        statistics.addFileProcessor(processor1);
        statistics.addFileProcessor(processor2);

        FileProcessor removed = statistics.removeFileProcessor("Source File");
        FileProcessor query = statistics.getFileProcessor("Text File");

        List processors = statistics.getAllFileProcessors();
        statistics.clearAllFileProcessors();

        // create a file statistics instance from given namespace
        statistics = new ExtendedFileStatistics(NAMESPACE);

        // manipulate reporting
        StatsReporting original = statistics.getReporting();
        statistics.setReporting(reporting2);

        // manipulate match all flag
        if (!statistics.isMatchAll()) {
            statistics.setMatchAll(true);
        }

        // Process File
        // process single file
        statistics.processFile(new File("test_files/test.txt"));
        statistics.processFile(new File("test_files/test.java"));

        // process directory
        statistics.processDirectory(new File("test_files/dir"));
        statistics.processDirectory(new File("test_files/dir1"), true);

        // Generate Report
        // obtain report
        String report = statistics.getReporting().generateReport();

        // output report
        statistics.printReport();
        statistics.printReport(new File("test_files/output.html"));

        new File("test_files/output.html").delete();

        // reset reporting
        statistics.getReporting().reset(new File("test_files"));
    }

    /**
     * <p>
     * Demonstrates how to use this component in console mode.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testConsoleDemo() throws Exception {
        TestHelper.removeNamespace(NAMESPACE);

        String[] args = new String[] {
            "-c", "main_config1.xml", "-n", NAMESPACE, "-b", "test_files", "-r",
            "test_files\\dir"
        };

        // a report should be output to console
        Main.main(args);

        TestHelper.loadConfigFile("config5.xml", DEFAULT_NAMESPACE);

        args = new String[] {"test_files\\dir", "test_files\\dir1", "test_files\\test.txt"};

        // a report should be output to console
        Main.main(args);
    }
}
