/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics;

import com.topcoder.file.statistics.matcher.ExtensionMatcher;
import com.topcoder.file.statistics.processor.linecounter.CStyleLineCounter;
import com.topcoder.file.statistics.processor.linecounter.TextLineCounter;
import com.topcoder.file.statistics.reporting.BasicStatsReporting;
import com.topcoder.file.statistics.reporting.XmlStatsReporting;
import junit.framework.TestCase;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;


/**
 * <p>
 * Unit test cases for ExtendedFileStatistics.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class ExtendedFileStatisticsV20cTests extends TestCase {
    /**
     * The the base path of the test files.
     */
    private static final String BASE_PATH = "test_files";

    /**
     * The namespace for this test.
     */
    private static final String NAMESPACE = "com.topcoder.file.statistics.unittest";

    /**
     * An Metric instance to test.
     */
    private Metric line = null;

    /**
     * An ExtendedFileStatistics instance to test.
     */
    private ExtendedFileStatistics fileStatistics = null;

    /**
     * A FileProcessor instance to test.
     */
    private FileProcessor processor1 = new CStyleLineCounter("java", new ExtensionMatcher("java"));

    /**
     * A FileProcessor instance to test.
     */
    private FileProcessor processor2 = new TextLineCounter("text", new ExtensionMatcher("txt"));

    /**
     * Creates an ExtendedFileStatistics instance.
     *
     * @throws Exception to JUnit
     */
    protected void setUp() throws Exception {
        fileStatistics = new ExtendedFileStatistics();

        line = (Metric) processor1.getSupportedMetrics().get(0);
    }

    /**
     * <p>
     * Creates a ExtendedFileStatistics instance with given config namespace.
     * </p>
     *
     * @param configFile the config file
     * @param namespace the config namespace
     *
     * @return the ExtendedFileStatistics instance
     *
     * @throws Exception to JUnit
     */
    private ExtendedFileStatistics createExtendedFileStatistics(String configFile, String namespace)
        throws Exception {
        TestHelper.loadConfigFile(configFile, namespace);

        ExtendedFileStatistics statistics = new ExtendedFileStatistics(namespace);
        TestHelper.removeNamespace(namespace);

        return statistics;
    }


    /**
     * <p>
     * Test processFile method with null argument. NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProccessFile1() throws Exception {
        try {
            this.fileStatistics.processFile(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Test testtestProcessDirectory method.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProcessDirectory() throws Exception {
        fileStatistics = createExtendedFileStatistics("v20/config.xml", NAMESPACE);

        XmlStatsReporting reporting = (XmlStatsReporting) fileStatistics.getReporting();
        reporting.reset(new File(BASE_PATH, "v20/a"));

        fileStatistics.processDirectory(new File(BASE_PATH, "v20/a"), true);
        
        fileStatistics.printReport();
        
        
    }

    /**
     * <p>
     * Test processDirectory(File) with null argument. NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProcessDirectory1() throws Exception {
        try {
            fileStatistics.processDirectory(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Test processDirectory(File) with invalid dir. IllegalArgumentException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProcessDirectory2() throws Exception {
        try {
            fileStatistics.processDirectory(new File(BASE_PATH, "no_such_dir"));
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * <p>
     * Test processDirectory(File) with invalid dir. IllegalArgumentException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProcessDirectory3() throws Exception {
        try {
            fileStatistics.processDirectory(new File(BASE_PATH, "test.txt"));
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * <p>
     * Test processDirectory(File, boolean) with null argument. NullPointerException should be
     * thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProcessDirectory4() throws Exception {
        try {
            fileStatistics.processDirectory(null, false);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Test processDirectory(File, boolean) with invalid dir. IllegalArgumentException should be
     * thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProcessDirectory5() throws Exception {
        try {
            fileStatistics.processDirectory(new File(BASE_PATH, "no_such_dir"), false);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * <p>
     * Test processDirectory(File, boolean) with invalid dir. IllegalArgumentException should be
     * thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProcessDirectory6() throws Exception {
        try {
            fileStatistics.processDirectory(new File(BASE_PATH, "test.txt"), false);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * <p>
     * Test processDirectory(File).
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProcessDirectory7() throws Exception {
        fileStatistics.addFileProcessor(processor1);
        fileStatistics.addFileProcessor(processor2);

        XmlStatsReporting reporting = (XmlStatsReporting) fileStatistics.getReporting();
        reporting.reset(new File(BASE_PATH));

        fileStatistics.processDirectory(new File(BASE_PATH, "dir"));

        assertEquals("3 should be returned", 3L,
            reporting.getFileStats().get("dir\\test.java").getStat(line));
        assertEquals("3 should be returned", 5L,
            reporting.getFileStats().get("dir\\test.txt").getStat(line));
    }

    /**
     * <p>
     * Test processDirectory(File, boolean).
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProcessDirectory8() throws Exception {
        fileStatistics.addFileProcessor(processor1);
        fileStatistics.addFileProcessor(processor2);

        XmlStatsReporting reporting = (XmlStatsReporting) fileStatistics.getReporting();
        reporting.reset(new File(BASE_PATH));

        fileStatistics.processDirectory(new File(BASE_PATH, "dir"), true);

        assertEquals("3 should be returned", 3L,
            reporting.getFileStats().get("dir\\test.java").getStat(line));
        assertEquals("3 should be returned", 5L,
            reporting.getFileStats().get("dir\\test.txt").getStat(line));
    }

    /**
     * <p>
     * Test printReport().
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testPrintReport1() throws Exception {
        fileStatistics.addFileProcessor(processor1);
        fileStatistics.addFileProcessor(processor2);

        XmlStatsReporting reporting = (XmlStatsReporting) fileStatistics.getReporting();
        reporting.reset(new File(BASE_PATH));

        fileStatistics.processDirectory(new File(BASE_PATH, "dir"), true);

        fileStatistics.printReport();
    }

    /**
     * <p>
     * Test printReport(File).
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testPrintReport2() throws Exception {
        fileStatistics.addFileProcessor(processor1);
        fileStatistics.addFileProcessor(processor2);

        XmlStatsReporting reporting = (XmlStatsReporting) fileStatistics.getReporting();
        reporting.reset(new File(BASE_PATH));

        fileStatistics.processDirectory(new File(BASE_PATH, "dir"), true);

        File output = new File(BASE_PATH, "output");
        fileStatistics.printReport(output);

        char[] buffer = new char[20000];
        BufferedReader reader = new BufferedReader(new FileReader(output));
        int length = reader.read(buffer);

        assertEquals("the result is incorrect", reporting.generateReport(),
            new String(buffer, 0, length));

        output.delete();
    }

    /**
     * <p>
     * Test printReport(File) with null dir. NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testPrintReport3() throws Exception {
        try {
            fileStatistics.printReport(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Test printReport(File) with invalid dir. IOException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testPrintReport4() throws Exception {
        try {
            fileStatistics.printReport(new File(BASE_PATH));
            fail("IOException should be thrown");
        } catch (IOException e) {
            // success
        }
    }
}
