/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics;

import com.topcoder.file.statistics.matcher.AnyFileMatcher;
import com.topcoder.file.statistics.matcher.ExtensionMatcher;
import com.topcoder.file.statistics.processor.linecounter.CStyleLineCounter;
import com.topcoder.file.statistics.processor.linecounter.TextLineCounter;
import com.topcoder.file.statistics.reporting.XmlStatsReporting;
import junit.framework.TestCase;
import java.io.File;


/**
 * <p>
 * Unit test cases for FileStatistics.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class FileStatisticsV20cTests extends TestCase {
    /**
     * The the base path of the test files.
     */
    private static final String BASE_PATH = "test_files";

    /**
     * The namespace for this test.
     */
    private static final String NAMESPACE = "com.topcoder.file.statistics.unittest";

    /**
     * An Metric instance to test.
     */
    private Metric line = null;

    /**
     * An FileStatistics instance to test.
     */
    private FileStatistics fileStatistics = null;

    /**
     * A FileProcessor instance to test.
     */
    private FileProcessor processor1 = new CStyleLineCounter("java", new ExtensionMatcher("java"));

    /**
     * A FileProcessor instance to test.
     */
    private FileProcessor processor2 = new TextLineCounter("text", new AnyFileMatcher());

    /**
     * Creates an FileStatistics instance.
     *
     * @throws Exception to JUnit
     */
    protected void setUp() throws Exception {
        fileStatistics = new FileStatistics();

        line = (Metric) processor1.getSupportedMetrics().get(0);
    }




    /**
     * <p>
     * Creates a FileStatistics instance with given config namespace.
     * </p>
     *
     * @param configFile the config file
     * @param namespace the config namespace
     *
     * @return the ExtendedFileStatistics instance
     *
     * @throws Exception to JUnit
     */
    private FileStatistics createFileStatistics(String configFile, String namespace)
        throws Exception {
        TestHelper.loadConfigFile(configFile, namespace);

        FileStatistics statistics = new FileStatistics(namespace);
        TestHelper.removeNamespace(namespace);

        return statistics;
    }

    /**
     * <p>
     * Test processFile method with null argument. NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProccessFile1() throws Exception {
        try {
            this.fileStatistics.processFile(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Test processFile method.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProccessFile2() throws Exception {
        fileStatistics.addFileProcessor(processor1);
        fileStatistics.addFileProcessor(processor2);

        XmlStatsReporting reporting = (XmlStatsReporting) fileStatistics.getReporting();
        reporting.reset(new File(BASE_PATH));

        fileStatistics.processFile(new File(BASE_PATH, "test.java"));

        assertEquals("only processor1 should be used", 3L,
            reporting.getFileStats().get("test.java").getStat(line));

        fileStatistics.setMatchAll(true);
        reporting.reset(new File(BASE_PATH));
        fileStatistics.processFile(new File(BASE_PATH, "test.java"));

        assertEquals("both processors should be used", 9L,
            reporting.getFileStats().get("test.java").getStat(line));
    }

    /**
     * <p>
     * Test processFile method. Process multiple files.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testProccessFile3() throws Exception {
        fileStatistics = createFileStatistics("config3.xml", NAMESPACE);

        XmlStatsReporting reporting = (XmlStatsReporting) fileStatistics.getReporting();
        reporting.reset(new File(BASE_PATH));

        fileStatistics.processFile(new File(BASE_PATH, "test.java"));
        fileStatistics.processFile(new File(BASE_PATH, "test.txt"));
        fileStatistics.processFile(new File(BASE_PATH, "dir/test.java"));
        fileStatistics.processFile(new File(BASE_PATH, "dir/test.txt"));

        assertEquals("3 should be returned", 3L,
            reporting.getFileStats().get("test.java").getStat(line));
        assertEquals("3 should be returned", 5L,
            reporting.getFileStats().get("test.txt").getStat(line));
        assertEquals("3 should be returned", 3L,
            reporting.getFileStats().get("dir\\test.java").getStat(line));
        assertEquals("3 should be returned", 5L,
            reporting.getFileStats().get("dir\\test.txt").getStat(line));
    }
}
