/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics.accuracytests;

import java.io.File;

import junit.framework.Assert;
import junit.framework.TestCase;

import com.topcoder.file.statistics.FileProcessor;
import com.topcoder.file.statistics.FileStatistics;
import com.topcoder.file.statistics.reporting.XmlStatsReporting;


/**
 * <p>
 * Unit test cases for FileStatistics
 * </p>
 *
 * @author cucu
 * @version 1.0
 */
public class FileStatisticsAccuracyTests extends TestCase {

    /**
     * Object to be tested
     */
    private FileStatistics fs;

    /**
     * Set up for each test.
     */
    public void setUp() throws Exception  {
        TestHelper.unloadNamespace(TestHelper.FILE_STATISTICS_NAMESPACE);
        TestHelper.loadConfig(TestHelper.FILE_STATISTICS_CONFIG);
        fs = new FileStatistics(TestHelper.FILE_STATISTICS_NAMESPACE);
    }

    /**
     * Tear down for each test.
     */
    public void tearDown() throws Exception  {
        TestHelper.unloadNamespace(TestHelper.FILE_STATISTICS_NAMESPACE);
    }

    /**
     * Test that the configuration is correctly loaded from configuration manager
     */
    public void testConstructorFromConfig()  {
        assertTrue("MatchAll should be true", fs.isMatchAll());
        assertTrue("Reporting should be an instance of XmlStatsReporting",
                fs.getReporting() instanceof XmlStatsReporting);

        assertEquals("bad File Processor loaded", "Text",
                ((FileProcessor) fs.getAllFileProcessors().get(0)).getAlias());
        assertEquals("bad File Processor loaded", "Source",
                ((FileProcessor) fs.getAllFileProcessors().get(1)).getAlias());
    }

    /**
     * Test the getFileProcessor method
     */
    public void testGetFileProcessor()  {
        assertEquals("bad File Processor retrieved", "Text", fs.getFileProcessor("Text").getAlias());
        assertNull("null expeceted for non existant alias", fs.getFileProcessor("nonexistant"));
    }

    /**
     * Test RemoveFileProcessor method
     */
    public void testRemoveFileProcessor()  {
        fs.removeFileProcessor("Text");
        assertNull("File Processor not deleted", fs.getFileProcessor("Text"));
    }

    /**
     *  Test the clearAllFileProcessors method
     */
    public void testClear() {
        fs.clearAllFileProcessors();
        assertEquals("Clear is not removing all the elements", 0, fs.getAllFileProcessors().size());
    }

    /**
     * Test the processFile method with a file that should be processed by both processors
     */
    public void testProcessFile1() throws Exception  {
        fs.processFile(new File(TestHelper.CS_FILE));
        String report = fs.getReporting().generateReport();

        // Save to a file for visual inspection
        TestHelper.saveToFile("FileStatisticsResults/ProcessFile1.xml", report);

        assertTrue("Report doesn't seem to be right", report.indexOf("<line>") >= 0);
        assertTrue("Report doesn't seem to be right", report.indexOf("Source") >= 0);
        assertTrue("Report doesn't seem to be right", report.indexOf("Text") >= 0);
    }

    /**
     * Test the ProcessFile method with a file that should be processed just by the Text Processor
     */
    public void testProcessFile2() throws Exception {
        fs.processFile(new File(TestHelper.BIG_TXT_FILE));
        String report = fs.getReporting().generateReport();

        // Save to a file for visual inspection
        TestHelper.saveToFile("FileStatisticsResults/ProcessFile2.xml", report);

        assertTrue("Report doesn't seem to be right", report.indexOf("<line>") >= 0);
        assertTrue("Report doesn't seem to be right", report.indexOf("Source") < 0);
        assertTrue("Report doesn't seem to be right", report.indexOf("Text") >= 0);
      }

    /**
     *  Test the ProcessFile method with a file that matches both processors, but setting isMatchAll to false
     *  so that just the Text processor must process it
     */
    public void testProcessFile3() throws Exception {
        fs.setMatchAll(false);

        fs.processFile(new File(TestHelper.CS_FILE));
        String report = fs.getReporting().generateReport();

        // Save to a file for visual inspection
        TestHelper.saveToFile("FileStatisticsResults/ProcessFile3.xml", report);

        assertTrue("Report doesn't seem to be right", report.indexOf("<line>") >= 0);
        assertTrue("Report doesn't seem to be right", report.indexOf("Source") < 0);
        assertTrue("Report doesn't seem to be right", report.indexOf("Text") >= 0);

    }

    /**
     * Test the ProcessFile method with 4 files: one that doesn't match anything, one that just matches the text
     * processor, one that just matches the source processor and the last that matches both
     */
    public void testProcessFile4() throws Exception {

        fs.processFile(new File(TestHelper.CS_FILE));
        fs.processFile(new File(TestHelper.CPP_FILE));
        fs.processFile(new File(TestHelper.DLL_FILE));
        fs.processFile(new File(TestHelper.TXT_FILE));
        String report = fs.getReporting().generateReport();

        // Save to a file for visual inspection
        TestHelper.saveToFile("FileStatisticsResults/ProcessFile4.xml", report);

        assertTrue("Report doesn't seem to be right", report.indexOf("<line>") >= 0);
        assertTrue("Report doesn't seem to be right", report.indexOf("Source") >= 0);
        assertTrue("Report doesn't seem to be right", report.indexOf("Text") >= 0);

    }

}
