/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics.extractor;

import com.topcoder.util.config.ConfigManager;
import com.topcoder.file.statistics.ArchiveExtractor;
import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.TestHelper;
import com.topcoder.file.statistics.matcher.ExtensionMatcher;
import com.topcoder.util.config.Property;
import junit.framework.TestCase;
import java.io.File;


/**
 * <p>
 * Unit test cases for ArchiveExtractorBase.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class ArchiveExtractorBaseTests extends TestCase {

	/**
     * The namespace for this test.
     */
    private static final String NAMESPACE = "com.topcoder.file.statistics.extractor.unittest";

    /**
     * The config file for this test.
     */
    private static final String CONFIG_FILE = "extractor/extractor_config.xml";
    										   
	/**
     * The the base path of the test files.
     */
    private static final String BASE_PATH = "test_files/extractor";

    /**
     * An ArchiveExtractorBase instance to test.
     */
    private ArchiveExtractorBase extractor = null;

    /**
     * Creates an empty StatsResult instance.
     * 
     * @throws Exception to JUnit
     */
    protected void setUp() throws Exception {
    	TestHelper.loadConfigFile(CONFIG_FILE, NAMESPACE);
    	extractor = new DummyRARExtractor(new ExtensionMatcher("rar"), '?');    	
    }

    /**
     * Remove the configuration after testing.
     *
     * @throws Exception to JUnit
     */
    protected void tearDown() throws Exception {
        TestHelper.removeNamespace(NAMESPACE);
    }
       
    /**
     * <p>
     * Verifies the constructor - ArchiveExtractorBase(FileMatcher, char).
     * </p>
     */
    public void testArchiveExtractorBase1() {
        assertTrue("Unable to instantiate ArchiveExtractorBase.",
            extractor instanceof ArchiveExtractor);
        assertTrue("the matcher is incorrect", extractor.getFileMatcher() instanceof ExtensionMatcher);
        assertEquals("the path separator is incorrect", '?', extractor.getPathSeparator());
        
    }
    

    /**
     * <p>
     * Verifies the constructor - ArchiveExtractorBase(Property).
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testArchiveExtractorBase2() throws Exception {
    	
    	Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE, "dummy");
    	
    	extractor = new DummyRARExtractor(property);
    	assertTrue("Unable to instantiate ArchiveExtractorBase.",
            extractor instanceof ArchiveExtractor);
        assertTrue("the matcher is incorrect", extractor.getFileMatcher() instanceof ExtensionMatcher);
        assertEquals("the path separator is incorrect", '?', extractor.getPathSeparator());
                    
    }
    
    /**
     * <p>
     * Verifies the constructor - ArchiveExtractorBase(Property) with default separator.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testArchiveExtractorBase3() throws Exception {
    	
    	Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE, "dummy_no_separator");
    	
    	extractor = new DummyRARExtractor(property);
    	assertTrue("Unable to instantiate ArchiveExtractorBase.",
            extractor instanceof ArchiveExtractor);
        assertTrue("the matcher is incorrect", extractor.getFileMatcher() instanceof ExtensionMatcher);
        assertEquals("the path separator is incorrect", '>', extractor.getPathSeparator());
                    
    }

    /**
     * <p>
     * Verifies the constructor - ArchiveExtractorBase(FileMatcher, char) with null argument.
     * NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testArchiveExtractorBase4() throws Exception {
        try {
            new DummyRARExtractor(null, '?');
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }
    
    /**
     * <p>
     * Verifies the constructor - ArchiveExtractorBase(Property) with null argument.
     * NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testArchiveExtractorBase5() throws Exception {
        try {
            new DummyRARExtractor(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }    
    
    /**
     * <p>
     * Verifies the constructor - ExtensionMatcher(Property) with invalid property.
     * The property constains no matcher
     * ConfigurationException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testArchiveExtractorBase6() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "invalid1");

        try {
            new DummyRARExtractor(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }
    
    /**
     * <p>
     * Verifies the constructor - ExtensionMatcher(Property) with invalid property.
     * The property constains no class_name.
     * ConfigurationException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testArchiveExtractorBase7() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "invalid2");

        try {
            new DummyRARExtractor(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }    
    
    /**
     * <p>
     * Verifies the constructor - ExtensionMatcher(Property) with invalid property.
     * The property constains invalid matcher class.
     * ConfigurationException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testArchiveExtractorBase8() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "invalid3");

        try {
            new DummyRARExtractor(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }   
    
    /**
     * <p>
     * Verifies the constructor - ExtensionMatcher(Property) with invalid property.
     * The property constains invalid separator.
     * ConfigurationException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testArchiveExtractorBase9() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "invalid4");

        try {
            new DummyRARExtractor(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }   
    
    /**
     * <p>
     * Tests the getFileMatcher method.
     * </p>
     */
    public void testGetFileMatcher() {
        assertTrue("the matcher is incorrect", extractor.getFileMatcher() instanceof ExtensionMatcher);
    }
    
    /**
     * <p>
     * Tests the getPathSeparator method.
     * </p>
     */
    public void testGetPathSeparator() {
        assertEquals("the path separator is incorrect", '?', extractor.getPathSeparator());
    }
    
    /**
     * <p>
     * Tests the extract method.
     * Argument is null, NullPointerException should be thrown.
     * </p>
     * 
     * @throws Exception to JUnit
     */
    public void testExtract1() throws Exception {
        try {
        	extractor.extract(null, new File(BASE_PATH));
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }    	
        
    }

    /**
     * <p>
     * Tests the extract method.
     * Argument is null, NullPointerException should be thrown.
     * </p>
     * 
     * @throws Exception to JUnit
     */
    public void testExtract2() throws Exception {
        try {
        	extractor.extract(new File(BASE_PATH, "foo.rar"), null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }    	
        
    }
    
    /**
     * <p>
     * Tests the extract method. Dummy file should be created
     * </p>
     * 
     * @throws Exception to JUnit
     */
    public void testExtract3() throws Exception {
        extractor.extract(new File(BASE_PATH, "foo.rar"), new File(BASE_PATH));
        assertTrue("the file should be created", new File(BASE_PATH, "dummy.file").exists());        
    }    

}

