/*
 * $(#)XmlStatsReportingTests.java		Sep 6, 2005
 * 
 * Copyright (c) 2005, TopCoder Inc. All rights reserved.
 */

package com.topcoder.file.statistics.failuretests.reporting;

import java.io.File;

import junit.framework.TestCase;

import com.topcoder.file.statistics.*;
import com.topcoder.file.statistics.matcher.ExtensionMatcher;
import com.topcoder.file.statistics.processor.linecounter.TextLineCounter;
import com.topcoder.file.statistics.reporting.XmlStatsReporting;
import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.Property;

/**
 * This test case contains the failure tests for the <code>XmlStatsReporting</code>
 * class.
 *
 * @author MPhk
 * @version 1.0
 */
public class XmlStatsReportingTests extends TestCase {

    /**
     * The the base path of the test files.
     */
    private static final String BASE_PATH = "test_files/failuretests/reporting";

    /**
     * The namespace for this test.
     */
    private static final String NAMESPACE = "com.topcoder.file.statistics.reporting.failuretests";

    /**
     * The config file for this test.
     */
    private static final String CONFIG_FILE = "failuretests/reporting/config.xml";

    /**
     * An XmlStatsReporting instance to test.
     */
    private XmlStatsReporting reporting = null;

    /**
     * A Metric instance to test.
     */
    private Metric metric1 = new Metric("metric1", "metric1", "foo");

    /**
     * A Metric instance to test.
     */
    private Metric metric2 = new Metric("metric2", "metric2", "foo");

    /**
     * A StatsResult instance to test.
     */
    private StatsResult result = null;

    /**
     * A FileProcessor instance to test.
     */
    private FileProcessor processor = new TextLineCounter("text", new ExtensionMatcher("txt"));

    /**
     * Sets up this test.
     *
     * @throws Exception in case anything goes wrong
     */
    protected void setUp() throws Exception {
        reporting = new XmlStatsReporting();
        result = new StatsResult();
        result.setStat(metric1, 10);
        result.setStat(metric2, 20);

        // clear the namespace first
        ConfigManager configManager = ConfigManager.getInstance();
        if (configManager.existsNamespace(NAMESPACE)) {
            configManager.removeNamespace(NAMESPACE);
        }
        // load the configuration
        ConfigManager.getInstance().add(CONFIG_FILE);
    }

    /**
     * Tears down this test.
     *
     * @throws Exception in case anything goes wrong
     */
    protected void tearDown() throws Exception {
        ConfigManager configManager = ConfigManager.getInstance();
        if (configManager.existsNamespace(NAMESPACE)) {
            configManager.removeNamespace(NAMESPACE);
        }
    }

    /**
     * Checks whether the constructor XmlStatsReporting(Property) properly throws
     * NullPointerException for null argument.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testXmlStatsReporting1() throws Exception {
        try {
            new XmlStatsReporting((Property) null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * Checks whether the constructor XmlStatsReporting(Property) properly throws
     * ConfigurationException for invalid property.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testXmlStatsReporting2() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE, "invalid1");

        try {
            new XmlStatsReporting(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }

    /**
     * Checks whether the constructor XmlStatsReporting(Property) properly throws
     * ConfigurationException for invalid property.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testXmlStatsReporting3() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE, "invalid2");

        try {
            new XmlStatsReporting(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }

    /**
     * Checks whether the constructor XmlStatsReporting(File) properly throws
     * NullPointerException for null argument.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testXmlStatsReporting4() throws Exception {
        try {
            new XmlStatsReporting((File) null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * Checks whether the constructor XmlStatsReporting(File) properly throws
     * IllegalArgumentException for invalid file argument.
     */
    public void testXmlStatsReporting5() {
        try {
            new XmlStatsReporting(new File(BASE_PATH));
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * Checks whether the constructor XmlStatsReporting(File) properly throws
     * IllegalArgumentException for invalid file argument.
     */
    public void testXmlStatsReporting6() {
        try {
            new XmlStatsReporting(new File(BASE_PATH, "inexistent_file"));
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * Checks whether the method aggregate(FileProcessor, File, StatsResult) properly throws
     * NullPointerException for null argument.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testAggregate1() throws Exception {
        try {
            reporting.aggregate(null, new File("file").getAbsolutePath(), result);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * Checks whether the method aggregate(FileProcessor, File, StatsResult) properly throws
     * NullPointerException for null argument.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testAggregate2() throws Exception {
        try {
            reporting.aggregate(processor, null, result);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * Checks whether the method aggregate(FileProcessor, File, StatsResult) properly throws
     * NullPointerException for null argument.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testAggregate3() throws Exception {
        try {
            reporting.aggregate(processor, new File("file").getAbsolutePath(), null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * Checks whether the method aggregate(FileProcessor, File, StatsResult) properly throws
     * StatsReportingException for invalid file argument.
     */
    public void testAggregate4() {
        reporting.reset(new File(BASE_PATH));

        try {
            reporting.aggregate(processor, new File(BASE_PATH).getAbsolutePath(), result);
            fail("StatsReportingException should be thrown");
        } catch (StatsReportingException e) {
            // success
        }
    }

    /**
     * Checks whether the method aggregate(FileProcessor, File, StatsResult) properly throws
     * StatsReportingException for invalid file argument.
     */
    public void testAggregate5() {
        reporting.reset(new File(BASE_PATH));

        try {
            reporting.aggregate(processor, new File(BASE_PATH + "invalid_file").getAbsolutePath(), result);
            fail("StatsReportingException should be thrown");
        } catch (StatsReportingException e) {
            // success
        }
    }

    /**
     * Checks whether the method aggregate(FileProcessor, File, StatsResult) properly throws
     * StatsReportingException for invalid file argument.
     */
    public void testAggregate6() {
        reporting.reset(new File(BASE_PATH));

        try {
            reporting.aggregate(processor, new File(new File(BASE_PATH).getParentFile(), "file.txt").getAbsolutePath(), result);
            fail("StatsReportingException should be thrown");
        } catch (StatsReportingException e) {
            // success
        }
    }
    
    /**
     * Checks whether the method reset(File) properly throws
     * IllegalArgumentException for invalid folder argument.
     */
    public void testReset() {
        try {
            File base = new File("INVALID_DIR");
            reporting.reset(base);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

}
