/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.TestHelper;
import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.Property;
import junit.framework.TestCase;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;


/**
 * <p>
 * Unit test cases for TypeMimeMatcher.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class MimeMatcherTests extends TestCase {
    /**
     * The namespace for this test.
     */
    private static final String NAMESPACE = "com.topcoder.file.statistics.matcher.unittest";

    /**
     * The config file for this test.
     */
    private static final String CONFIG_FILE = "matcher/matcher_config.xml";

    /**
     * The namespace for Magic Numbers component.
     */
    private static final String MAGIC_NUMBERS_NAMESPACE = "com.topcoder.file.mime.MagicNumbers.xml";

    /**
     * The the base path of the test files.
     */
    private static final String BASE_PATH = "test_files/matcher/";

    /**
     * An TypeNameMatcher instance to test.
     */
    private MimeMatcher matcher = null;

    /**
     * Load the configuration before testing.
     *
     * @throws Exception to JUnit
     */
    protected void setUp() throws Exception {
        TestHelper.loadConfigFile(CONFIG_FILE, NAMESPACE);
        TestHelper.removeNamespace(MAGIC_NUMBERS_NAMESPACE);
        matcher = new MimeMatcher("image/gif");
    }

    /**
     * Remove the configuration after testing.
     *
     * @throws Exception to JUnit
     */
    protected void tearDown() throws Exception {
        TestHelper.removeNamespace(NAMESPACE);
        TestHelper.removeNamespace(MAGIC_NUMBERS_NAMESPACE);
    }

    /**
     * <p>
     * Verifies the constructor - MimeMatcher(String).
     * </p>
     */
    public void testMimeMatcher1() {
        assertTrue("Unable to instantiate MimeMatcher.",
            new MimeMatcher("foo") instanceof FileTypeMatcher);
    }

    /**
     * <p>
     * Verifies the constructor - MimeMatcher(String) with null argument. NullPointerException
     * should be thrown.
     * </p>
     */
    public void testMimeMatcher2() {
        try {
            new MimeMatcher((String) null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - MimeMatcher(Property).
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMimeMatcher3() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "mimes_matcher");
        assertTrue("Unable to instantiate MimeMatcher.",
            new MimeMatcher(property) instanceof FileTypeMatcher);
    }

    /**
     * <p>
     * Verifies the constructor - MimeMatcher(Property) with null argument.
     * NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMimeMatcher4() throws Exception {
        try {
            new MimeMatcher((Property) null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - MimeMatcher(Property) with invalid property.
     * ConfigurationException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMimeMatcher5() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "mimes_matcher_invalid1");

        try {
            new MimeMatcher(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - MimeMatcher(Property) with invalid property.
     * ConfigurationException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMimeMatcher6() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "mimes_matcher_invalid2");

        try {
            new MimeMatcher(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - MimeMatcher(List).
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMimeMatcher7() throws Exception {
        List extensions = new ArrayList();
        extensions.add("foo");
        extensions.add("bar");
        assertTrue("Unable to instantiate MimeMatcher.",
            new MimeMatcher(extensions) instanceof FileTypeMatcher);
    }

    /**
     * <p>
     * Verifies the constructor - MimeMatcher(List) with null argument. NullPointerException
     * should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMimeMatcher8() throws Exception {
        try {
            new MimeMatcher((List) null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - MimeMatcher(List) with empty list. IllegalArgumentException
     * should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMimeMatcher9() throws Exception {
        try {
            new MimeMatcher(new ArrayList());
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - MimeMatcher(List) with invalid list.
     * IllegalArgumentException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMimeMatcher10() throws Exception {
        List extensions = new ArrayList();
        extensions.add("foo");
        extensions.add(new ArrayList());

        try {
            new MimeMatcher(extensions);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - MimeMatcher(List) with invalid list.
     * IllegalArgumentException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMimeMatcher11() throws Exception {
        List extensions = new ArrayList();
        extensions.add("foo");
        extensions.add("");

        try {
            new MimeMatcher(extensions);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - MimeMatcher(List) with invalid list.
     * IllegalArgumentException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMimeMatcher12() throws Exception {
        List extensions = new ArrayList();
        extensions.add("foo");
        extensions.add(null);

        try {
            new MimeMatcher(extensions);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - MimeMatcher(Property) with invalid property.
     * ConfigurationException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMimeMatcher13() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "mimes_matcher_invalid3");

        try {
            new MimeMatcher(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the match method with null argument. NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMatch1() throws Exception {
        try {
            matcher.match((File) null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the match method. GIF file should be accepted.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMatch2() throws Exception {

        assertTrue("The result should be true", matcher.match(new File(BASE_PATH, "test.GIF")));
    }
    /**
     * <p>
     * Tests the match method. BMP file should not be accepted.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMatch3() throws Exception {
        assertFalse("The result should be false", matcher.match(new File(BASE_PATH, "test.BMP")));
    }
    /**
     * <p>
     * Tests the match method. JPG file should not be accepted.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMatch4() throws Exception {
        assertFalse("The result should be false", matcher.match(new File(BASE_PATH, "test.JPG")));
    }

    /**
     * <p>
     * Tests the match method of multi-mime matcher.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMatch5() throws Exception {
        // create a matcher that accepts GIF and JPEG files.
        List mimes = new ArrayList();
        mimes.add("image/gif");
        mimes.add("image/jpeg");
        matcher = new MimeMatcher(mimes);

        assertTrue("The result should be true", matcher.match(new File(BASE_PATH, "test.JPG")));

    }

    /**
     * <p>
     * Tests the match method of multi-mime matcher.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMatch6() throws Exception {
        // create a matcher that accepts GIF and JPEG files.
        List mimes = new ArrayList();
        mimes.add("image/gif");
        mimes.add("image/jpeg");
        matcher = new MimeMatcher(mimes);

        assertTrue("The result should be true", matcher.match(new File(BASE_PATH, "test.GIF")));

    }

    /**
     * <p>
     * Tests the match method of multi-mime matcher.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMatch7() throws Exception {
        // create a matcher that accepts GIF and JPEG files.
        List mimes = new ArrayList();
        mimes.add("image/gif");
        mimes.add("image/jpeg");
        matcher = new MimeMatcher(mimes);

        assertFalse("The result should be false", matcher.match(new File(BASE_PATH, "test.BMP")));

    }

    /**
     * <p>
     * Tests the match method of multi-mime matcher.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMatch8() throws Exception {
        // create a matcher that accepts GIF and JPEG files.
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "mimes_matcher");
        matcher = new MimeMatcher(property);

        assertTrue("The result should be true", matcher.match(new File(BASE_PATH, "test.JPG")));
    }

    /**
     * <p>
     * Tests the match method of multi-mime matcher.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMatch9() throws Exception {
        // create a matcher that accepts GIF and JPEG files.
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "mimes_matcher");
        matcher = new MimeMatcher(property);

        assertTrue("The result should be true", matcher.match(new File(BASE_PATH, "test.GIF")));
    }

    /**
     * <p>
     * Tests the match method of multi-mime matcher.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMatch10() throws Exception {
        // create a matcher that accepts GIF and JPEG files.
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "mimes_matcher");
        matcher = new MimeMatcher(property);

        assertFalse("The result should be false", matcher.match(new File(BASE_PATH, "test.BMP")));
    }

    /**
     * <p>
     * Tests the match method with invalid argument. IOException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMatch11() throws Exception {
        try {
            matcher.match(new File(BASE_PATH, "no_this_file"));
            fail("IOException should be thrown");
        } catch (IOException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the match method with invalid argument. IOException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMatch12() throws Exception {
        try {
            matcher.match(new File(BASE_PATH));
            fail("IOException should be thrown");
        } catch (IOException e) {
            // success
        }
    }
}
