/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics.reporting;

import com.topcoder.file.statistics.FileProcessor;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsReporting;
import com.topcoder.file.statistics.StatsReportingException;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.file.statistics.matcher.ExtensionMatcher;
import com.topcoder.file.statistics.processor.linecounter.CStyleLineCounter;
import com.topcoder.file.statistics.processor.linecounter.TextLineCounter;
import com.topcoder.util.config.Property;
import junit.framework.TestCase;
import java.io.File;


/**
 * <p>
 * Unit test cases for BasicStatsReporting.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class BasicStatsReportingTests extends TestCase {
    /**
     * The the base path of the test files.
     */
    private static final String BASE_PATH = "test_files/reporting";

    /**
     * An BasicStatsReporting instance to test.
     */
    private BasicStatsReporting reporting = null;

    /**
     * A Metric instance to test.
     */
    private Metric metric1 = new Metric("metric1", "metric1", "foo");

    /**
     * A Metric instance to test.
     */
    private Metric metric2 = new Metric("metric2", "metric2", "foo");

    /**
     * A Metric instance to test.
     */
    private Metric metric3 = new Metric("metric3", "metric3", "foo");

    /**
     * A StatsResult instance to test.
     */
    private StatsResult result1 = null;

    /**
     * A StatsResult instance to test.
     */
    private StatsResult result2 = null;

    /**
     * A FileProcessor instance to test.
     */
    private FileProcessor processor1 = new TextLineCounter("text", new ExtensionMatcher("txt"));

    /**
     * A FileProcessor instance to test.
     */
    private FileProcessor processor2 = new CStyleLineCounter("java", new ExtensionMatcher("java"));

    /**
     * Creates an empty StatsResult instance.
     */
    protected void setUp() {
        reporting = new BasicStatsReporting();

        result1 = new StatsResult();
        result2 = new StatsResult();

        result1.setStat(metric1, 10);
        result1.setStat(metric2, 20);
        result2.setStat(metric2, 30);
        result2.setStat(metric3, 0);
    }

    /**
     * <p>
     * Verifies the constructor - BasicStatsReporting(String).
     * </p>
     */
    public void testBasicStatsReporting1() {
        assertTrue("Unable to instantiate BasicStatsReporting.",
            new BasicStatsReporting() instanceof StatsReporting);
    }

    /**
     * <p>
     * Verifies the constructor - BasicStatsReporting(Property).
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testBasicStatsReporting2() throws Exception {
        assertTrue("Unable to instantiate BasicStatsReporting.",
            new BasicStatsReporting(new Property("foo")) instanceof StatsReporting);
    }

    /**
     * <p>
     * Verifies the constructor - BasicStatsReporting(Property) with null argument.
     * NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testBasicStatsReporting3() throws Exception {
        try {
            new BasicStatsReporting(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the getBaseDirectory method.
     * </p>
     */
    public void testGetBaseDirectory1() {
        assertNull("null is expected", reporting.getBaseDirectory());
    }

    /**
     * <p>
     * Tests the getBaseDirectory method.
     * </p>
     */
    public void testGetBaseDirectory2() {
        File base = new File(BASE_PATH);
        reporting.reset(base);
        assertEquals("base is expected", base, reporting.getBaseDirectory());
    }

    /**
     * <p>
     * Tests the aggregate method with base directory.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testAggregate1() throws Exception {
        reporting.reset(new File(BASE_PATH));
        reporting.aggregate(processor1, new File(BASE_PATH, "a/b.txt").getAbsolutePath(), result1);
        reporting.aggregate(processor2, new File(BASE_PATH, "a/b.txt").getAbsolutePath(), result2);
        reporting.aggregate(processor1, new File(BASE_PATH, "a/c.txt").getAbsolutePath(), result1);
        reporting.aggregate(processor2, new File(BASE_PATH, "b/a.txt").getAbsolutePath(), result2);

        assertTrue("file stats is incorrect",
            reporting.getFileStats().get("a\\b.txt").getStat(metric1) == 10);
        assertTrue("file stats is incorrect",
            reporting.getFileStats().get("a\\b.txt").getStat(metric2) == 50);
        assertTrue("file stats is incorrect",
            reporting.getFileStats().get("a\\b.txt").getStat(metric3) == 0);

        assertTrue("dir stats is incorrect",
            reporting.getDirectoryStats().get("a").getStat(metric1) == 20);
        assertTrue("dir stats is incorrect",
            reporting.getDirectoryStats().get("a").getStat(metric2) == 70);
        assertTrue("dir stats is incorrect",
            reporting.getDirectoryStats().get("a").getStat(metric3) == 0);

        assertTrue("alias stats is incorrect",
            reporting.getAliasStats().get("text").getStat(metric1) == 20);
        assertTrue("alias stats is incorrect",
            reporting.getAliasStats().get("text").getStat(metric2) == 40);

        assertTrue("overall stats is incorrect", reporting.getOverallStats().getStat(metric1) == 20);
        assertTrue("overall stats is incorrect", reporting.getOverallStats().getStat(metric2) == 100);
        assertTrue("overall stats is incorrect", reporting.getOverallStats().getStat(metric3) == 0);
    }

    /**
     * <p>
     * Tests the aggregate method without base directory.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testAggregate2() throws Exception {
        reporting.aggregate(processor1, new File(BASE_PATH, "a/b.txt").getAbsolutePath(), result1);
        reporting.aggregate(processor2, new File(BASE_PATH, "a/b.txt").getAbsolutePath(), result2);
        reporting.aggregate(processor1, new File(BASE_PATH, "a/c.txt").getAbsolutePath(), result1);

        String file = new File(BASE_PATH, "a/b.txt").getAbsolutePath();
        assertTrue("file stats is incorrect",
            reporting.getFileStats().get(file).getStat(metric1) == 10);
        assertTrue("file stats is incorrect",
            reporting.getFileStats().get(file).getStat(metric2) == 50);
        assertTrue("file stats is incorrect",
            reporting.getFileStats().get(file).getStat(metric3) == 0);

        String dir = new File(BASE_PATH, "a").getAbsolutePath();
        assertTrue("dir stats is incorrect",
            reporting.getDirectoryStats().get(dir).getStat(metric1) == 20);
        assertTrue("dir stats is incorrect",
            reporting.getDirectoryStats().get(dir).getStat(metric2) == 70);
        assertTrue("dir stats is incorrect",
            reporting.getDirectoryStats().get(dir).getStat(metric3) == 0);
    }

    /**
     * <p>
     * Tests the aggregate method with null argument. NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testAggregate3() throws Exception {
        try {
            reporting.aggregate(null, new File("foo").getAbsolutePath(), result1);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the aggregate method with null argument. NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testAggregate4() throws Exception {
        try {
            reporting.aggregate(processor1, null, result1);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the aggregate method with null argument. NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testAggregate5() throws Exception {
        try {
            reporting.aggregate(processor1, new File("foo").getAbsolutePath(), null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the aggregate method with invalid file. StatsReportingException should be thrown.
     * </p>
     */
    public void testAggregate6() {
        reporting.reset(new File(BASE_PATH));

        try {
            reporting.aggregate(processor1, new File(BASE_PATH).getAbsolutePath(), result1);
            fail("StatsReportingException should be thrown");
        } catch (StatsReportingException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the aggregate method with invalid file. StatsReportingException should be thrown.
     * </p>
     */
    public void testAggregate7() {
        reporting.reset(new File(BASE_PATH));

        try {
            reporting.aggregate(processor1, new File(BASE_PATH + "foo").getAbsolutePath(), result1);
            fail("StatsReportingException should be thrown");
        } catch (StatsReportingException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the aggregate method with invalid file. StatsReportingException should be thrown.
     * </p>
     */
    public void testAggregate8() {
        reporting.reset(new File(BASE_PATH));

        try {
            reporting.aggregate(processor1, new File(BASE_PATH).getParentFile().getAbsolutePath(), result1);
            fail("StatsReportingException should be thrown");
        } catch (StatsReportingException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the reset method.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testReset() throws Exception {
        File base = new File(BASE_PATH);
        reporting.reset(base);
        assertEquals("base is expected", base, reporting.getBaseDirectory());

        reporting.aggregate(processor1, new File(BASE_PATH, "a/b.txt").getAbsolutePath(), result1);

        reporting.reset(null);
        assertNull("base is expected", reporting.getBaseDirectory());

        assertNull("file stats is incorrect", reporting.getFileStats().get("a\\b.txt"));
        assertNull("dir stats is incorrect", reporting.getDirectoryStats().get("a"));
        assertNull("alias stats is incorrect", reporting.getAliasStats().get("text"));
    }

    /**
     * <p>
     * Tests the getFileStats method.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testGetFileStats() throws Exception {
        reporting.reset(new File(BASE_PATH));
        reporting.aggregate(processor1, new File(BASE_PATH, "a/b.txt").getAbsolutePath(), result1);

        assertTrue("file stats is incorrect",
            reporting.getFileStats().get("a\\b.txt").getStat(metric1) == 10);
        assertTrue("file stats is incorrect",
            reporting.getFileStats().get("a\\b.txt").getStat(metric2) == 20);
    }

    /**
     * <p>
     * Tests the getDirectoryStats method.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testGetDirectoryStats() throws Exception {
        reporting.reset(new File(BASE_PATH));
        reporting.aggregate(processor1, new File(BASE_PATH, "a/b.txt").getAbsolutePath(), result1);

        assertTrue("dir stats is incorrect",
            reporting.getDirectoryStats().get("a").getStat(metric1) == 10);
        assertTrue("dir stats is incorrect",
            reporting.getDirectoryStats().get("a").getStat(metric2) == 20);
    }

    /**
     * <p>
     * Tests the getAliasStats method.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testGetAliasStats() throws Exception {
        reporting.reset(new File(BASE_PATH));
        reporting.aggregate(processor1, new File(BASE_PATH, "a/b.txt").getAbsolutePath(), result1);

        assertTrue("alias stats is incorrect",
            reporting.getAliasStats().get("text").getStat(metric1) == 10);
        assertTrue("alias stats is incorrect",
            reporting.getAliasStats().get("text").getStat(metric2) == 20);
    }

    /**
     * <p>
     * Tests the getOverallStats method.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testGetOverallStats() throws Exception {
        reporting.reset(new File(BASE_PATH));
        reporting.aggregate(processor1, new File(BASE_PATH, "a/b.txt").getAbsolutePath(), result1);

        assertTrue("overall stats is incorrect", reporting.getOverallStats().getStat(metric1) == 10);
        assertTrue("overall", reporting.getOverallStats().getStat(metric2) == 20);
    }

    /**
     * <p>
     * Tests the generateReport method. See test_files/reporting/sample_report.txt for more
     * details.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testGenerateReport() throws Exception {
        reporting.reset(new File(BASE_PATH));
        reporting.aggregate(processor1, new File(BASE_PATH, "a/b.txt").getAbsolutePath(), result1);
        reporting.aggregate(processor2, new File(BASE_PATH, "a/b.txt").getAbsolutePath(), result2);
        reporting.aggregate(processor1, new File(BASE_PATH, "a/c.txt").getAbsolutePath(), result1);
        reporting.aggregate(processor2, new File(BASE_PATH, "b/a.txt").getAbsolutePath(), result2);

        String report = reporting.generateReport();

        assertTrue("the report is rendered incorrectly.", report.indexOf("a\\b.txt") != -1);
        assertTrue("the report is rendered incorrectly.", report.indexOf("a\\c.txt") != -1);
        assertTrue("the report is rendered incorrectly.", report.indexOf("b\\a.txt") != -1);

        assertTrue("the report is rendered incorrectly.", report.indexOf("text") != -1);
        assertTrue("the report is rendered incorrectly.", report.indexOf("java") != -1);

        assertTrue("the report is rendered incorrectly.", report.indexOf("metric1") != -1);
        assertTrue("the report is rendered incorrectly.", report.indexOf("metric2") != -1);
        assertTrue("the report is rendered incorrectly.", report.indexOf("metric3") != -1);
    }
}
