/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics.reporting;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileProcessor;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsReporting;
import com.topcoder.file.statistics.StatsReportingException;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.file.statistics.TestHelper;
import com.topcoder.file.statistics.extractor.ZipExtractor;
import com.topcoder.file.statistics.matcher.ExtensionMatcher;
import com.topcoder.file.statistics.processor.linecounter.CStyleLineCounter;
import com.topcoder.file.statistics.processor.linecounter.TextLineCounter;
import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.Property;
import junit.framework.TestCase;
import java.io.File;


/**
 * <p>
 * Unit test cases for XmlStatsReporting.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class XmlStatsReportingV20cTests extends TestCase {
	
	/**
     * The namespace for this test.
     */
    private static final String NAMESPACE = "com.topcoder.file.statistics.reporting.unittest";

    /**
     * The config file for this test.
     */
    private static final String CONFIG_FILE = "reporting/reporting_config.xml";    										  
    
    /**
     * The the base path of the test files.
     */
    private static final String BASE_PATH = "test_files/reporting";

    /**
     * An XmlStatsReporting instance to test.
     */
    private XmlStatsReporting reporting = null;

    /**
     * A Metric instance to test.
     */
    private Metric metric1 = new Metric("metric1", "metric1", "foo");

    /**
     * A Metric instance to test.
     */
    private Metric metric2 = new Metric("metric2", "metric2", "foo");

    /**
     * A Metric instance to test.
     */
    private Metric metric3 = new Metric("metric3", "metric3", "foo");

    /**
     * A StatsResult instance to test.
     */
    private StatsResult result1 = null;

    /**
     * A StatsResult instance to test.
     */
    private StatsResult result2 = null;

    /**
     * A FileProcessor instance to test.
     */
    private FileProcessor processor1 = new TextLineCounter("text", new ExtensionMatcher("txt"));

    /**
     * A FileProcessor instance to test.
     */
    private FileProcessor processor2 = new CStyleLineCounter("java", new ExtensionMatcher("java"));

    /**
     * Creates an empty StatsResult instance.
     * 
     * @throws Exception to JUnit
     */
    protected void setUp() throws Exception {
    	
    	TestHelper.loadConfigFile(CONFIG_FILE, NAMESPACE);
    	
        reporting = new XmlStatsReporting();

        result1 = new StatsResult();
        result2 = new StatsResult();

        result1.setStat(metric1, 10);
        result1.setStat(metric2, 20);
        result2.setStat(metric2, 30);
        result2.setStat(metric3, 0);
    }
    
    /**
     * Remove the configuration after testing.
     *
     * @throws Exception to JUnit
     */
    protected void tearDown() throws Exception {
        TestHelper.removeNamespace(NAMESPACE);
   
    }
    
    /**
     * <p>
     * Verifies the constructor - XmlStatsReporting(Property).
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testXmlStatsReporting1() throws Exception {
    	
    	Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE, "reporting");
    	
    	reporting = new XmlStatsReporting(property);
    	assertTrue("Unable to instantiate ZipExtractor.",
            reporting instanceof StatsReporting);
        assertEquals("the path separator is incorrect", "?z", reporting.getPathSeparators());
                    
    }
    
    /**
     * <p>
     * Verifies the constructor - XmlStatsReporting(Property) with default separator.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testXmlStatsReporting2() throws Exception {
    	
    	Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE, "reporting_no_separator");
    	
    	reporting = new XmlStatsReporting(property);
    	assertTrue("Unable to instantiate ZipExtractor.",
            reporting instanceof StatsReporting);
        assertEquals("the path separator is incorrect", ">", reporting.getPathSeparators());
                    
    }

    /**
     * <p>
     * Verifies the constructor - XmlStatsReporting(FileMatcher, char) with null argument.
     * NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testXmlStatsReporting3() throws Exception {
        try {
            new ZipExtractor(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }
    
    /**
     * <p>
     * Verifies the constructor - XmlStatsReporting(Property) with invalid property.
     * ConfigurationException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testZipExtractor4() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "invalid");

        try {
            new XmlStatsReporting(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }    

    /**
     * <p>
     * Tests the aggregate method with base directory.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testAggregate1() throws Exception {
        reporting.reset(new File(BASE_PATH));
        reporting.aggregate(processor1, new File(BASE_PATH, "a/b.txt").getAbsolutePath(), result1);
        reporting.aggregate(processor2, new File(BASE_PATH, "a/b.txt").getAbsolutePath(), result2);
        reporting.aggregate(processor1, new File(BASE_PATH, "a/c.txt").getAbsolutePath(), result1);
        reporting.aggregate(processor2, new File(BASE_PATH, "b/a.txt").getAbsolutePath(), result2);

        assertTrue("file stats is incorrect",
            reporting.getFileStats().get("a\\b.txt").getStat(metric1) == 10);
        assertTrue("file stats is incorrect",
            reporting.getFileStats().get("a\\b.txt").getStat(metric2) == 50);
        assertTrue("file stats is incorrect",
            reporting.getFileStats().get("a\\b.txt").getStat(metric3) == 0);

        assertTrue("dir stats is incorrect",
            reporting.getDirectoryStats().get("a").getStat(metric1) == 20);
        assertTrue("dir stats is incorrect",
            reporting.getDirectoryStats().get("a").getStat(metric2) == 70);
        assertTrue("dir stats is incorrect",
            reporting.getDirectoryStats().get("a").getStat(metric3) == 0);

        assertTrue("alias stats is incorrect",
            reporting.getAliasStats().get("text").getStat(metric1) == 20);
        assertTrue("alias stats is incorrect",
            reporting.getAliasStats().get("text").getStat(metric2) == 40);

        assertTrue("overall stats is incorrect", reporting.getOverallStats().getStat(metric1) == 20);
        assertTrue("overall stats is incorrect", reporting.getOverallStats().getStat(metric2) == 100);
        assertTrue("overall stats is incorrect", reporting.getOverallStats().getStat(metric3) == 0);
    }

    /**
     * <p>
     * Tests the aggregate method without base directory.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testAggregate2() throws Exception {
        reporting.aggregate(processor1, new File(BASE_PATH, "a/b.txt").getAbsolutePath(), result1);
        reporting.aggregate(processor2, new File(BASE_PATH, "a/b.txt").getAbsolutePath(), result2);
        reporting.aggregate(processor1, new File(BASE_PATH, "a/c.txt").getAbsolutePath(), result1);

        String file = new File(BASE_PATH, "a/b.txt").getAbsolutePath();
        assertTrue("file stats is incorrect",
            reporting.getFileStats().get(file).getStat(metric1) == 10);
        assertTrue("file stats is incorrect",
            reporting.getFileStats().get(file).getStat(metric2) == 50);
        assertTrue("file stats is incorrect",
            reporting.getFileStats().get(file).getStat(metric3) == 0);

        String dir = new File(BASE_PATH, "a").getAbsolutePath();
        assertTrue("dir stats is incorrect",
            reporting.getDirectoryStats().get(dir).getStat(metric1) == 20);
        assertTrue("dir stats is incorrect",
            reporting.getDirectoryStats().get(dir).getStat(metric2) == 70);
        assertTrue("dir stats is incorrect",
            reporting.getDirectoryStats().get(dir).getStat(metric3) == 0);
    }

    /**
     * <p>
     * Tests the aggregate method with null argument. NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testAggregate3() throws Exception {
        try {
            reporting.aggregate(null, new File("foo").getAbsolutePath(), result1);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the aggregate method with null argument. NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testAggregate4() throws Exception {
        try {
            reporting.aggregate(processor1, null, result1);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the aggregate method with null argument. NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testAggregate5() throws Exception {
        try {
            reporting.aggregate(processor1, new File("foo").getAbsolutePath(), null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the aggregate method with invalid file. StatsReportingException should be thrown.
     * </p>
     */
    public void testAggregate6() {
        reporting.reset(new File(BASE_PATH));

        try {
            reporting.aggregate(processor1, new File(BASE_PATH).getAbsolutePath(), result1);
            fail("StatsReportingException should be thrown");
        } catch (StatsReportingException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the aggregate method with invalid file. StatsReportingException should be thrown.
     * </p>
     */
    public void testAggregate7() {
        reporting.reset(new File(BASE_PATH));

        try {
            reporting.aggregate(processor1, new File(BASE_PATH + "foo").getAbsolutePath(), result1);
            fail("StatsReportingException should be thrown");
        } catch (StatsReportingException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the aggregate method with invalid file. StatsReportingException should be thrown.
     * </p>
     */
    public void testAggregate8() {
        reporting.reset(new File(BASE_PATH));

        try {
            reporting.aggregate(processor1, new File(BASE_PATH).getParentFile().getAbsolutePath(), result1);
            fail("StatsReportingException should be thrown");
        } catch (StatsReportingException e) {
            // success
        }
    }
    
    /**
     * <p>
     * Tests the getPathSeparators method.
     * </p>
     */
    public void testGetPathSeparators() {
    	assertEquals("the path separators is invalid", ">", reporting.getPathSeparators());
    }
    
    /**
     * <p>
     * Tests the setPathSeparators method.
     * </p>
     */
    public void testSetPathSeparators1() {
    	reporting.setPathSeparators("abc");
    	assertEquals("the path separators is invalid", "abc", reporting.getPathSeparators());
    }
    
    /**
     * <p>
     * Tests the setPathSeparators method with null.
     * </p>
     */
    public void testSetPathSeparators2() {
        try {
        	reporting.setPathSeparators(null);            
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the setPathSeparators method with null.
     * </p>
     */
    public void testSetPathSeparators3() {
        try {
        	reporting.setPathSeparators(">>");            
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    	
}
